/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "InkLevelView.h"
InkLevelView::InkLevelView(BRect frame,const char *name,int inknum)
:BView(frame,name,B_FOLLOW_NONE,B_WILL_DRAW)
,InkLevelGroup(inknum)
{
  SetViewColor(B_TRANSPARENT_32_BIT);
}
int InkLevelView::ViewWidth(int numinks)
{
  return (numinks * (InkLevelViewSpace::width + InkLevelViewSpace::space)) + InkLevelViewSpace::space;
}
void InkLevelView::Draw(BRect rect)
{
  BRect bound;
  int32 ink_count;
  BRect lrect,l_item_rect;
  BRect prect,p_item_rect;    
  // Fill Background
  SetHighColor(216,216,216);
  bound = Bounds();
  FillRect(bound);
  // Draw Border with caption
  DrawBox(bound);
      
  // Retrieve number of inks
  ink_count = CountInks();
  int height = (int) bound.Height() - InkLevelViewSpace::space;
  lrect = BRect(0,InkLevelViewSpace::space,InkLevelViewSpace::width,height - (InkLevelViewSpace::gap + InkLevelViewSpace::pattern_height));
  prect = BRect(0,lrect.bottom + InkLevelViewSpace::gap,InkLevelViewSpace::width,height);
  l_item_rect = lrect;
  p_item_rect = prect;
  l_item_rect.OffsetBy(InkLevelViewSpace::space,0);
  p_item_rect.OffsetBy(InkLevelViewSpace::space,0);
  for (int i = 0;i < ink_count;i++)
  {
    DrawLevel(i,l_item_rect);
    DrawPattern(i,p_item_rect);
    l_item_rect.OffsetBy(lrect.Width() + InkLevelViewSpace::space,0);
    p_item_rect.OffsetBy(prect.Width() + InkLevelViewSpace::space,0);
  }   
}
void InkLevelView::DrawBox(BRect rect)
{  
  int color;
  color = 128;
  // Draw TOP Horizon and Left Vertical
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.right,rect.top));
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.left,rect.bottom));
  // Draw Bottom Horizon and Right Vertical
  color = 255;
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.right,rect.top),BPoint(rect.right,rect.bottom));
  StrokeLine(BPoint(rect.right,rect.bottom),BPoint(rect.left,rect.bottom));

  rect.right--;
  rect.bottom--;
  rect.left++;
  rect.top++;
  
  color = 255;
  // Draw TOP Horizon and Left Vertical
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.right,rect.top));
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.left,rect.bottom));
  // Draw Bottom Horizon and Right Vertical
  color = 128;
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.right,rect.top),BPoint(rect.right,rect.bottom));
  StrokeLine(BPoint(rect.right,rect.bottom),BPoint(rect.left,rect.bottom));
}

void InkLevelView::DrawLevel(int index,BRect rect)
{
  DrawInBoxLevel(index,rect);
}
void InkLevelView::DrawPattern(int index,BRect rect)
{
  rgb_color color;
  ItemColor(index,&color);
  DrawInBox(rect,color);
}
void InkLevelView::DrawInBox(BRect rect,rgb_color bgcolor)
{
  int color;
  color = 128;
  // Draw TOP Horizon and Left Vertical
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.right,rect.top));
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.left,rect.bottom));
  // Draw Bottom Horizon and Right Vertical
  color = 255;
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.right,rect.top),BPoint(rect.right,rect.bottom));
  StrokeLine(BPoint(rect.right,rect.bottom),BPoint(rect.left,rect.bottom));
  
  rect.right--;
  rect.bottom--;
  rect.left++;
  rect.top++;

  SetHighColor(bgcolor);
  FillRect(rect);  
}
void InkLevelView::DrawInBoxLevel(int index,BRect rect)
{
  int color;
  color = 128;
  // Draw TOP Horizon and Left Vertical
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.right,rect.top));
  StrokeLine(BPoint(rect.left,rect.top),BPoint(rect.left,rect.bottom));
  // Draw Bottom Horizon and Right Vertical
  color = 255;
  SetHighColor(color,color,color);
  StrokeLine(BPoint(rect.right,rect.top),BPoint(rect.right,rect.bottom));
  StrokeLine(BPoint(rect.right,rect.bottom),BPoint(rect.left,rect.bottom));
  
  rect.right--;
  rect.bottom--;
  rect.left++;
  rect.top++;
  
  FillLevel(index,rect);
}
void InkLevelView::FillLevel(int index,BRect rect)
{
  rgb_color color;
  if (ItemLevel(index) != 0.0f)
  {
    int height = (int) rect.Height();
    rect.top = rect.top + (height - (height * ItemLevel(index)));
    ItemColor(index,&color);
    SetHighColor(color);
    FillRect(rect);
  }
}
