/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "JobSetupWindow.h"
#include "DebugUtils.h"
JobSetupWindow::JobSetupWindow(const char *title,PrinterAddon *addon,PrinterInfo *printer_info,BMessage *msg)
:BWindow(CenterOfScreen(JobSetupWindowSpace::DEF_WIDTH,JobSetupWindowSpace::DEF_HEIGHT),title,B_TITLED_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_DRAW)
,fsem(-1),faddon(addon),fprinter_info(printer_info),fmsg(msg),funit(printer_info->Unit())
{
  fsem = create_sem(0,"Jobsetupwindow");
  BuildGUI();
  tb_view->Select(0);
}
void JobSetupWindow::Quit()
{
  delete_sem(fsem);
  BWindow::Quit();
}
void JobSetupWindow::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case JobSetupWindowSpace::MSGCODE_PAGE_ALL_PAGE:
    OnEventAllPage();
    break;
    
    case JobSetupWindowSpace::MSGCODE_PAGE_SEL_PAGE:
    OnEventSelPage();
    break;
    
    case JobSetupWindowSpace::MSGCODE_NUM_OF_COPIES:
    OnEventCopyNum();
    break;
    
    case JobSetupWindowSpace::MSGCODE_COLLATE_ORDER:
    OnEventCollateOrder();
    break;
    
    case JobSetupWindowSpace::MSGCODE_REVERSE_ORDER:
    OnEventReverseOrder();
    break;
    
    case JobSetupWindowSpace::MSGCODE_PAPERTYPE:
    OnEventPaperType();
    break;
    
    case JobSetupWindowSpace::MSGCODE_COLOR:
    OnEventColorMode();
    break;
    
    case JobSetupWindowSpace::MSGCODE_QUALITY_SCALE:
    OnEventQualityScale();
    break;
    
    case JobSetupWindowSpace::MSGCODE_PRINT:
    OnBtnPrint();
    break;
 
    case JobSetupWindowSpace::MSGCODE_PREVIEW:
    OnBtnPreview();
    break;
    
    case JobSetupWindowSpace::MSGCODE_DISCARD:
    OnBtnDiscard();
    break;
    
    case JobSetupWindowSpace::MSGCODE_NOZZLE_CHECK:
    OnBtnNozzleCheck();
    break;

    case JobSetupWindowSpace::MSGCODE_HEAD_CLEAN:
    OnBtnHeadClean();
    break;

    case JobSetupWindowSpace::MSGCODE_CARTRIDGE_CHANGE:
    OnBtnChangeCartridge();
    break;

    case JobSetupWindowSpace::MSGCODE_UPDATE_INKLEVEL:
    OnBtnUpdateInkLevel();
    break;

    default:
    BWindow::MessageReceived(msg);
  };
}
void JobSetupWindow::Go()
{
  Lock();
  SetTo(fmsg);
  UpdateFunctionAvailability();
  Show();
  Unlock();

  acquire_sem(fsem);
  Lock();
  Quit();
}

void JobSetupWindow::BuildGUI()
{
  BRect rect_bg_box,rect_sub_box;
  BRect r,r1,r2;
  BRect bound = Bounds();
  bound.bottom++;
  bound.right++; 
  
  /* ============================================================= */
  // adding mainbox
  BBox *mainbox = new BBox(bound,NULL);
  mainbox->SetBorder(B_NO_BORDER);
  AddChild(mainbox);
  /* ============================================================= */

  /* ============================================================= */
  // create tab view
  BRect tb_rect = BRect(5,5,bound.Width() - 5,bound.Height() - 51);
  tb_view = new BTabView(tb_rect,"tabview");
  mainbox->AddChild(tb_view);

  // adjusting tab view RECT  
  BRect tb_sub_rect = tb_view->Bounds();
  tb_sub_rect.bottom -= tb_view->TabHeight() + 2;
  tb_sub_rect.right -= 2;
  /* ============================================================= */

  /* ============================================================= */
  // create print format tab view
  BTab* rtab = new BTab();
  BBox *bg_box = new BBox(tb_sub_rect);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(JobSetupWindowSpace::LB_TAB_PRINTJOB);
  
  // create print page group
  rect_bg_box = bg_box->Bounds();
  r = BRect(10,10,230,85);
  BBox *sub_box = new BBox(r,NULL);
  sub_box->SetLabel(JobSetupWindowSpace::LB_GROUP_PAGES);
  bg_box->AddChild(sub_box);
  rect_sub_box = sub_box->Bounds();
  r1 = BRect(10,20,70,30);
  rb_all_page = new BRadioButton(r1,NULL,JobSetupWindowSpace::LB_ALL_PAGE,new BMessage(JobSetupWindowSpace::MSGCODE_PAGE_ALL_PAGE));
  sub_box->AddChild(rb_all_page);
  r1.InsetBy(0,25);
  rb_sel_page = new BRadioButton(r1,NULL,JobSetupWindowSpace::LB_SEL_PAGE,new BMessage(JobSetupWindowSpace::MSGCODE_PAGE_SEL_PAGE));
  sub_box->AddChild(rb_sel_page);
  r1.left = r1.right + 5;
  r1.right = r1.left + 40;
  tv_page_start = new BTextControl(r1,NULL,NULL,NULL,new BMessage(JobSetupWindowSpace::MSGCODE_PAGE_SEL_START));
  tv_page_start->TextView()->SetMaxBytes(3);
  for (uint32 c = 0;c < 255;c++)
  {
    if (c >= '0' && c <= '9')
    tv_page_start->TextView()->AllowChar(c);
    else
    tv_page_start->TextView()->DisallowChar(c);
  }  
  sub_box->AddChild(tv_page_start);

  r1.left = r1.right + 5;
  r1.right = r1.left + 20 + 40;

  tv_page_end = new BTextControl(r1,NULL,JobSetupWindowSpace::LB_TO,NULL,new BMessage(JobSetupWindowSpace::MSGCODE_PAGE_SEL_END));
  tv_page_end->SetDivider(20);
  tv_page_end->SetAlignment(B_ALIGN_CENTER,B_ALIGN_LEFT);
  tv_page_end->TextView()->SetMaxBytes(3);
  for (uint32 c = 0;c < 255;c++)
  {
    if (c >= '0' && c <= '9')
    tv_page_end->TextView()->AllowChar(c);
    else
    tv_page_end->TextView()->DisallowChar(c);
  }  
  sub_box->AddChild(tv_page_end);

  // make copies group
  r.top = r.bottom + 10;
  r.bottom = r.top + 100;
  sub_box = new BBox(r);
  rect_sub_box = sub_box->Bounds();
  sub_box->SetLabel(JobSetupWindowSpace::LB_GROUP_COPIES);
  bg_box->AddChild(sub_box);

  // adding copies bitmap view  
  r1 = BRect(10,15,114,65);
  bv_copies = new PageOrderView(r1);
  sub_box->AddChild(bv_copies);
  
  r2 = BRect(r1.right + 10,20,r1.right + 80,35);
  // adding textcontrol for # copies
  tf_copies = new BTextControl(r2,NULL,JobSetupWindowSpace::LB_COPIES,"1",new BMessage(JobSetupWindowSpace::MSGCODE_NUM_OF_COPIES));
  tf_copies->SetDivider(be_plain_font->StringWidth(JobSetupWindowSpace::LB_COPIES));
  tf_copies->TextView()->SetMaxBytes(2);
  for (uint32 c = 0;c < 255;c++)
  {
    if (c >= '0' && c <= '9')
    tf_copies->TextView()->AllowChar(c);
    else
    tf_copies->TextView()->DisallowChar(c);
  }  
  sub_box->AddChild(tf_copies);

  r2.right += 20;  
  r2.top += 25;
  r2.bottom += 25;

  // adding collate copies option
  chk_collate = new BCheckBox(r2,NULL,JobSetupWindowSpace::LB_COLLATE,new BMessage(JobSetupWindowSpace::MSGCODE_COLLATE_ORDER));
  sub_box->AddChild(chk_collate);

  r2.top += 25;
  r2.bottom += 25;

  chk_reverse = new BCheckBox(r2,NULL,JobSetupWindowSpace::LB_REVERSE,new BMessage(JobSetupWindowSpace::MSGCODE_REVERSE_ORDER));
  sub_box->AddChild(chk_reverse);     
  // add printer preview view
  r = BRect(r.right + 10,10,rect_bg_box.Width() - 8,135);
  BBox *tempbox = new BBox(r);
  bg_box->AddChild(tempbox);
  r = tempbox->Bounds();
  r.left += 2;
  r.right -= 2;
  r.top += 2;
  r.bottom -= 2;
    
  bv_style = new PagePrintStyleView(r);
  tempbox->AddChild(bv_style);

  /* ============================================================= */

  /* ============================================================= */
  // create position tab view
  rtab = new BTab();
  bg_box = new BBox(tb_sub_rect,NULL);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(JobSetupWindowSpace::LB_TAB_OPTION);
  
  // Create paper type group 
  r1 = BRect(10,10,rect_bg_box.Width() - 160,80);
  sub_box = new BBox(r1);
  rect_sub_box = sub_box->Bounds();
  sub_box->SetLabel(JobSetupWindowSpace::LB_PAPERTYPE_GROUP);
  bg_box->AddChild(sub_box);
  // create paper type menu
  mn_paper_type = new BPopUpMenu("");
  BMenuField *mf_paper_type = new BMenuField(BRect(10,25,rect_sub_box.Width() - 10,50),NULL,"",mn_paper_type);
  mf_paper_type->SetDivider(0);
  mf_paper_type->SetAlignment(B_ALIGN_LEFT);
  sub_box->AddChild(mf_paper_type);  
  // load independence setting
  LoadPaperType();
  
  // Create paper type group 
  r1 = BRect(rect_bg_box.Width() - 150,10,rect_bg_box.Width() - 10,80);
  sub_box = new BBox(r1);
  rect_sub_box = sub_box->Bounds();
  sub_box->SetLabel(JobSetupWindowSpace::LB_MODE_COLOR_GROUP);
  bg_box->AddChild(sub_box);

  r1.left = 10;
  r1.right = 10 + 32;
  r1.top  = (rect_sub_box.Height() - 32) / 2;
  r1.bottom = r1.top + 32;
    
  bv_color_mode = new PrintingModeView(r1,false);
  sub_box->AddChild(bv_color_mode);
  
  r2 = BRect(r1.right + 10,15,rect_sub_box.Width() - 10,40);
  rb_mode_color = new BRadioButton(r2,NULL,JobSetupWindowSpace::LB_MODE_COLOR_COLOR,new BMessage(JobSetupWindowSpace::MSGCODE_COLOR));
  sub_box->AddChild(rb_mode_color);
  r2.InsetBy(0,25);

  rb_mode_mono = new BRadioButton(r2,NULL,JobSetupWindowSpace::LB_MODE_COLOR_MONO,new BMessage(JobSetupWindowSpace::MSGCODE_COLOR));
  sub_box->AddChild(rb_mode_mono);
  
  r1 = BRect(10,85,rect_bg_box.Width() - 10,rect_bg_box.Height() - 10);
  f_quality_box = new BBox(r1);
  rect_sub_box = f_quality_box->Bounds();
  f_quality_box->SetLabel(JobSetupWindowSpace::LB_QUALITY_GROUP);
  bg_box->AddChild(f_quality_box);
  
  r1 = BRect(10,25,78,68 + 25);
  bv_quality = new PrintingQualityView(r1,2,0);
  f_quality_box->AddChild(bv_quality);
 
  r2 = f_quality_rect = BRect(r1.right + 40,20,r1.right + 120 + 40,65);
  sd_quality = NULL;
  
  // add resolution string view
  r2.top = r2.bottom;
  r2.bottom = r2.top + 20;
  r2.right = rect_sub_box.Width() - 10;
  lb_resolution = new ResolutionView(r2);
  lb_resolution->SetTo(360,360);
  f_quality_box->AddChild(lb_resolution);
  /* ============================================================= */

  /* ============================================================= */
  // create Utilities tab view
  rtab = new BTab();
  bg_box = new BBox(tb_sub_rect,NULL);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(JobSetupWindowSpace::LB_TAB_UTILITIES);
  rect_bg_box = bg_box->Bounds();
  /* ============================================================= */

  // Adding Snooze Check button
  int gap = 18;
  int extend_space = 3;
  r1 = BRect(10,10,be_plain_font->StringWidth("  ") + be_plain_font->StringWidth(JobSetupWindowSpace::LB_CHANGE_CARTRIDGE) + 10 + 10,30);
  r2 = BRect(r1.right + 10,r1.top - extend_space,rect_bg_box.Width() - 10,r1.bottom + extend_space);
  btn_nozzle_check = new BButton(r1,NULL,JobSetupWindowSpace::LB_NOZZLE_CHECK,new BMessage(JobSetupWindowSpace::MSGCODE_NOZZLE_CHECK));
  bg_box->AddChild(btn_nozzle_check);
  lb_nozzle_check = new MultiLinesStringView(r2,NULL,JobSetupWindowSpace::LB_NOZZLE_CHECK_DETAIL);
  bg_box->AddChild(lb_nozzle_check);
  
  // Adding Snooze Clean button
  r1.OffsetBy(0,r1.Height() + gap);
  r2 = BRect(r1.right + 10,r1.top - extend_space,rect_bg_box.Width() - 10,r1.bottom + extend_space);
  btn_head_clean = new BButton(r1,NULL,JobSetupWindowSpace::LB_HEAD_CLEAN,new BMessage(JobSetupWindowSpace::MSGCODE_HEAD_CLEAN));
  bg_box->AddChild(btn_head_clean);
  lb_head_clean = new MultiLinesStringView(r2,NULL,JobSetupWindowSpace::LB_HEAD_CLEAN_DETAIL);
  bg_box->AddChild(lb_head_clean);
  
  // Adding Ink Eject button
  r1.OffsetBy(0,r1.Height() + gap);
  r2 = BRect(r1.right + 10,r1.top - extend_space,rect_bg_box.Width() - 10,r1.bottom + extend_space);
  btn_change_cartridge = new BButton(r1,NULL,JobSetupWindowSpace::LB_CHANGE_CARTRIDGE,new BMessage(JobSetupWindowSpace::MSGCODE_CARTRIDGE_CHANGE));
  bg_box->AddChild(btn_change_cartridge);
  lb_change_cartridge = new MultiLinesStringView(r2,NULL,JobSetupWindowSpace::LB_CHANGE_CARTRIDGE_DETAIL);
  bg_box->AddChild(lb_change_cartridge);
  
  // Adding Ink Level View
  r1 = BRect(r1.left,r1.bottom + 15 ,r2.right,rect_bg_box.Height() - gap);
  BBox *ink_box = new BBox(r1,JobSetupWindowSpace::LB_INK_BOX);
  ink_box->SetLabel(JobSetupWindowSpace::LB_INK_BOX);
  bg_box->AddChild(ink_box);
  rect_bg_box = ink_box->Bounds();
  r1 = BRect(10,10 + 5,0,rect_bg_box.Height() - 10);
  r1.right = r1.left + InkLevelView::ViewWidth(faddon->NumberOfInks());
  inklevel = new InkLevelView(r1,NULL,faddon->NumberOfInks());
  ink_box->AddChild(inklevel);
  r2 = BRect(r1.right + 10,r1.top,rect_bg_box.Width() - 10,r1.top + 20);
  lb_ink_level = new BStringView(r2,NULL,NULL);
  ink_box->AddChild(lb_ink_level);  
  r2 = BRect(r1.right + 10,r2.bottom + 5,r1.right + 10,r2.bottom + 5 + 20);
  
  r2.right = r2.left + be_bold_font->StringWidth(JobSetupWindowSpace::LB_UPDATE_INKLEVEL) + 10;
  if (r2.right > rect_bg_box.Width() - 10)
  r2.right = rect_bg_box.Width() - 10;
  
  btn_ink_level = new BButton(r2,JobSetupWindowSpace::LB_UPDATE_INKLEVEL,JobSetupWindowSpace::LB_UPDATE_INKLEVEL,new BMessage(JobSetupWindowSpace::MSGCODE_UPDATE_INKLEVEL));
  ink_box->AddChild(btn_ink_level);
  /* ============================================================= */
  
  /* ============================================================= */
  // Adding buttons 
  bound = mainbox->Bounds();
  BRect btn_rect = BRect(8,bound.Height() - 36,80,bound.Height() - 18);
  btn_discard = new BButton(btn_rect,NULL,JobSetupWindowSpace::LB_DISCARD,new BMessage(JobSetupWindowSpace::MSGCODE_DISCARD));
  mainbox->AddChild(btn_discard);

  btn_rect.left = bound.Width() - 170;
  btn_rect.right = btn_rect.left + 80;

  btn_preview = new BButton(btn_rect,NULL,JobSetupWindowSpace::LB_PREVIEW,new BMessage(JobSetupWindowSpace::MSGCODE_PREVIEW));
  mainbox->AddChild(btn_preview);

  btn_rect.left = bound.Width() - 80;
  btn_rect.right = bound.Width() - 10;
  btn_print = new BButton(btn_rect,NULL,JobSetupWindowSpace::LB_PRINT,new BMessage(JobSetupWindowSpace::MSGCODE_PRINT));
  btn_print->MakeDefault(true);
  mainbox->AddChild(btn_print);       
  /* ============================================================= */
}
void JobSetupWindow::OnBtnPrint()
{
  Lock();
  fmsg->what = 'okok';
  Retrieve(fmsg);
  SetPreview(false);
  BString str;
  release_sem(fsem);
  Unlock();
}
void JobSetupWindow::OnBtnPreview()
{
  Lock();
  fmsg->what = 'okok';
  Retrieve(fmsg);
  SetPreview(true);
  release_sem(fsem);
  Unlock();
}
void JobSetupWindow::OnBtnDiscard()
{
  Lock();
  if (fmsg != NULL)
  {
    fmsg->what = 'baad';
  }
  release_sem(fsem);
  Unlock();
}
void JobSetupWindow::OnBtnHeadClean()
{
  Lock();
  faddon->HeadClean(fprinter_info);
  Unlock();
}
void JobSetupWindow::OnBtnNozzleCheck()
{
  Lock();
  faddon->NozzleCheck(fprinter_info);
  Unlock();
}
void JobSetupWindow::OnBtnChangeCartridge()
{
  Lock();
  faddon->ChangeCartridge(fprinter_info);
  Unlock();
}
void JobSetupWindow::OnBtnUpdateInkLevel()
{
  Lock();
  printer_utility_return_t ret;
  ret = faddon->InkLevel(fprinter_info,inklevel);
  if (ret == PRINTER_OK)
  {
    inklevel->Invalidate();
    UpdateInkLevelStatus("Status : Ready");
  }
  else if (ret == PRINTER_ERROR_COMMUNICATION)
  UpdateInkLevelStatus("Status : Unable to communicate to the printer");
  else
  UpdateInkLevelStatus("Status : Printer is currently busy");
  Unlock();
}

void JobSetupWindow::LoadPaperType()
{
  BMenuItem *newitem;
  support_paper_media_st *temp_paper_media;
  if ((temp_paper_media = faddon->SupportPaperMedia()) != NULL)
  {
    while (temp_paper_media->id >= 0)
    {
      newitem = new BMenuItem(temp_paper_media->name,new BMessage(JobSetupWindowSpace::MSGCODE_PAPERTYPE));
      mn_paper_type->AddItem(newitem);  
      temp_paper_media++;
    }
  }
  if (mn_paper_type->CountItems() == 0)
  {
    newitem = new BMenuItem("Plain paper",new BMessage(JobSetupWindowSpace::MSGCODE_PAPERTYPE));
    mn_paper_type->AddItem(newitem);  
    temp_paper_media++;
  }
}
void JobSetupWindow::SetTo(BMessage *msg)
{
  BString str;

  /* ========================================================== */
  int32 first_page,last_page;
  if ((msg->FindInt32(MSGSTORE_NAME_FIRST_PAGE,&first_page) < B_OK) || (first_page < 1))
  first_page = 1;
  // last page 
  if (msg->FindInt32(MSGSTORE_NAME_LAST_PAGE,&last_page) < B_OK)
  last_page = LONG_MAX;
  
  if (first_page == 1)
  {
    if (last_page == LONG_MAX)
    {
      // all page
      rb_all_page->SetValue(true);
      tv_page_start->SetText("");
      tv_page_end->SetText("");
      SetEnableSelPageField(false);
    }      
    else
    {
      // specify page range
      rb_sel_page->SetValue(true);
      str = "";
      str << (int32) first_page;    
      tv_page_start->SetText(str.String());
      str = "";
      str << (int32) last_page;    
      tv_page_end->SetText(str.String());
      SetEnableSelPageField(true);
    }
  }
  else
  {
    // specify page range
    rb_sel_page->SetValue(true);
    str = "";
    str << (int32) first_page;    
    tv_page_start->SetText(str.String());
    // specify page range
    if (last_page < LONG_MAX)
    {
      // to some page
      str = "";
      str << (int32) last_page;    
      tv_page_end->SetText(str.String());
    }
    else
    {
      // to last page
      tv_page_end->SetText("");
    }
    SetEnableSelPageField(true);
  }
  /* ========================================================== */


  /* ==================================================================== */
  // # copies
  int32 p_copies;
  if (msg->FindInt32(MSGSTORE_NAME_COPIES,&p_copies) == B_OK)
  {
    if (p_copies < 0)
    p_copies = 1;
    else if (p_copies > 99)
    p_copies = 99;
  }
  else
  p_copies = 1;

  BString str_copies;
  str_copies << (int32) p_copies;
  tf_copies->SetText(str_copies.String());
  // -------------------------------------------------------
    
  // Collate Order
  bool p_collate_order;
  if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&p_collate_order) != B_OK)
  {
    p_collate_order = false;
    msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,p_collate_order);
  }
  chk_collate->SetValue(p_collate_order);
  // -------------------------------------------------------

  // Reverse Order
  bool p_reverse_order;
  if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&p_reverse_order) != B_OK)
  {
    p_reverse_order = false;
    msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,p_reverse_order);
  }
  chk_reverse->SetValue(p_reverse_order);
  // -------------------------------------------------------
  bv_copies->SetTo(p_collate_order,p_reverse_order);
  /* ==================================================================== */


  /* ==================================================================== */
  // orientation
  int32 p_orientation;
  if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&p_orientation) == B_OK)
  {
    if ((p_orientation != MSGSTORE_ORIENTATION_PORTRAIT) && (p_orientation != MSGSTORE_ORIENTATION_LANDSCAPE))
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
  }
  else
  {
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
    msg->AddInt32(MSGSTORE_NAME_ORIENTATION,p_orientation);
  }
  // -------------------------------------------------------
  
  // printable area style
  bool normal_printable_area;
  if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&normal_printable_area) != B_OK)
  normal_printable_area = true;
  
  bv_style->SetTo(p_orientation == MSGSTORE_ORIENTATION_PORTRAIT,normal_printable_area);
  /* ==================================================================== */
  
  /* ==================================================================== */
  // Paper Kind
  int32 p_paper_media = 0;
  msg->FindInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,&p_paper_media);
  mn_paper_type->ItemAt(p_paper_media)->SetMarked(true);  
  /* ==================================================================== */
  
  /* ==================================================================== */
  bool p_color_printing = true;
  if (msg->FindBool(MSGSTORE_NAME_PRINTING_COLOR,&p_color_printing) < B_OK)
  p_color_printing = true;
  if (!faddon->SupportColorPrint())
  {
    p_color_printing = false;
    rb_mode_color->SetEnabled(false);
  }
  
  if (p_color_printing)
  rb_mode_color->SetValue(true);
  else
  rb_mode_mono->SetValue(true);
  bv_color_mode->ColorPrinter(p_color_printing);

  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  
  // Quality Index
  int32 quality_index;
  if (msg->FindInt32(MSGSTORE_NAME_QUALITY,&quality_index) < B_OK)
  quality_index = -1;
   
  // set the quality slider to this quality mode 
  if ((quality_index < 0) || (quality_index >= index[paper_type_index].count))
  {
    CreateSDQuality(index[paper_type_index].relative_def,index[paper_type_index].count);
    bv_quality->Range(index[paper_type_index].count - 1);
    SetLBQualityFromGUI();
    SetBVQualityFromGUI();
  }
  else
  {
    CreateSDQuality(quality_index,index[paper_type_index].count);
    bv_quality->Range(index[paper_type_index].count - 1);
    SetLBQualityFromGUI();
    SetBVQualityFromGUI();
  }
  /* ==================================================================== */
}
int32 JobSetupWindow::GetNumberOfCopies()
{
  int32 temp;
  if (sscanf(tf_copies->Text(),"%ld",&temp) < 0)
  return 1;
  return temp;
}
void JobSetupWindow::Retrieve(BMessage *msg)
{
  int32 int32_value;
  int64 int64_value;
//  float float_value;
  bool  bool_value;
  // pages
  int32 first_page;
  int32 last_page;
  /* ========================================================== */
  if (rb_all_page->Value())
  {
    first_page = 1;
    last_page  = INT_MAX;
  }
  else
  {
    if (sscanf(tv_page_start->Text(),"%ld",&first_page) < 0)
    first_page = 1;
    if (sscanf(tv_page_end->Text(),"%ld",&last_page) < 0)
    last_page = INT_MAX;
  }
  
  if (msg->FindInt32(MSGSTORE_NAME_FIRST_PAGE,&int32_value) < B_OK)
  msg->AddInt32(MSGSTORE_NAME_FIRST_PAGE,first_page);
  else
  msg->ReplaceInt32(MSGSTORE_NAME_FIRST_PAGE,first_page);
  
  if (msg->FindInt32(MSGSTORE_NAME_LAST_PAGE,&int32_value) < B_OK)
  msg->AddInt32(MSGSTORE_NAME_LAST_PAGE,last_page);
  else
  msg->ReplaceInt32(MSGSTORE_NAME_LAST_PAGE,last_page);
  /* ========================================================== */
  
  /* ========================================================== */
  // # copies
  if (msg->FindInt32(MSGSTORE_NAME_COPIES,&int32_value) == B_OK)
  msg->ReplaceInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
  else
  msg->AddInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
    // -------------------------------------------------------
    
  // Collate Order    
  if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&bool_value) == B_OK)
  msg->ReplaceBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
  else
  msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
  // -------------------------------------------------------

  // Reverse Order
  if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&bool_value) == B_OK)
  msg->ReplaceBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
  else
  msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
  // -------------------------------------------------------
  /* ========================================================== */
  // paper type
  int32 paper_type_index;
  paper_type_index = GetGUISelectPaperType();
  if (msg->FindInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,&int32_value) == B_OK)
  msg->ReplaceInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,paper_type_index);
  else
  msg->AddInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,paper_type_index);
  
  // color mode
  if (msg->FindBool(MSGSTORE_NAME_PRINTING_COLOR,&bool_value) == B_OK)
  msg->ReplaceBool(MSGSTORE_NAME_PRINTING_COLOR,rb_mode_color->Value());
  else
  msg->AddBool(MSGSTORE_NAME_PRINTING_COLOR,rb_mode_color->Value());
  
  // quality index
  if (msg->FindInt32(MSGSTORE_NAME_QUALITY,&int32_value) == B_OK)
  msg->ReplaceInt32(MSGSTORE_NAME_QUALITY,sd_quality->Value());
  else
  msg->AddInt32(MSGSTORE_NAME_QUALITY,sd_quality->Value());
  print_quality_st *qlist;
  print_quality_index_st *qindex;
  if (rb_mode_color->Value())
  {
    qlist = faddon->SupportQualityColor();
    qindex = faddon->SupportQualityIndexColor();
  } 
  else
  {
    qlist  = faddon->SupportQualityMono();
    qindex = faddon->SupportQualityIndexMono();
  }
  // resolution 
  int32 paper_size;
  if (msg->FindInt32(MSGSTORE_NAME_PAPER_SIZE,&paper_size) != B_OK)
  paper_size = PAGE_A4;
    
  int32 resindex = qindex[paper_type_index].offset + sd_quality->Value();
  int32 xres = qlist[resindex].xres;
  int32 yres = qlist[resindex].yres;

  // x resolution
  if (msg->FindInt64(MSGSTORE_NAME_RESOLUTION_X,&int64_value) == B_OK)
  msg->ReplaceInt64(MSGSTORE_NAME_RESOLUTION_X,xres);
  else
  msg->AddInt64(MSGSTORE_NAME_RESOLUTION_X,xres);

  // y resolution
  if (msg->FindInt64(MSGSTORE_NAME_RESOLUTION_Y,&int64_value) == B_OK)
  msg->ReplaceInt64(MSGSTORE_NAME_RESOLUTION_Y,yres);
  else
  msg->AddInt64(MSGSTORE_NAME_RESOLUTION_Y,yres);

  int64 max_of_res;
  if (xres > yres)
  max_of_res = xres;
  else
  max_of_res = yres;

/*  
  // first get info about paper_size,user_define_width,user_define_height and printstyle;
  float user_define_width,user_define_height;
  int32 print_style;
  BRect rect;
  if (paper_size == PAGE_USER_DEFINE)
  {
    // User Define Paper Width
    if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,&user_define_width) != B_OK)
    paper_size = PAGE_A4;
    else
    {
      if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,&user_define_height) != B_OK)
      paper_size = PAGE_A4;
    }
  }

  if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&print_style) != B_OK)
  print_style = MSGSTORE_ORIENTATION_PORTRAIT;
  else if ((print_style != MSGSTORE_ORIENTATION_PORTRAIT) && (print_style != MSGSTORE_ORIENTATION_LANDSCAPE))
  print_style = MSGSTORE_ORIENTATION_PORTRAIT;
  
  bool p_area_normal;
  
  // printable_style
  if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&p_area_normal) != B_OK)
  p_area_normal = true;
  
  // paper_rect
  BRect paper_rect = GetPrintRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
  BRect print_rect = paper_rect;
  
  paper_rect = BRect(0,0,paper_rect.Width(),paper_rect.Height());
  if (msg->FindRect(MSGSTORE_NAME_PAPER_RECT,&rect) == B_OK)
  msg->ReplaceRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);
  else
  msg->AddRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);

  // printable_rect
  if (msg->FindRect(MSGSTORE_NAME_PRINTABLE_RECT,&rect) == B_OK)
  msg->ReplaceRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);
  else
  msg->AddRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);

  // internal paper rect
  paper_rect = GetPaperRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
  if (msg->FindRect(MSGSTORE_NAME_INT_PAPER_RECT,&rect) == B_OK)
  msg->ReplaceRect(MSGSTORE_NAME_INT_PAPER_RECT,paper_rect);
  else
  msg->AddRect(MSGSTORE_NAME_INT_PAPER_RECT,paper_rect);
  
  // internal printable rect
  print_rect = GetPrintRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
  if (msg->FindRect(MSGSTORE_NAME_INT_PRINTABLE_RECT,&rect) == B_OK)
  msg->ReplaceRect(MSGSTORE_NAME_INT_PRINTABLE_RECT,print_rect);
  else
  msg->AddRect(MSGSTORE_NAME_INT_PRINTABLE_RECT,print_rect);

  // scaling 
  if (msg->FindFloat(MSGSTORE_NAME_SCALING,&float_value) == B_OK)
  msg->ReplaceFloat(MSGSTORE_NAME_SCALING,MSGSTORE_INTERNAL_SCALING_DEFAULT);
  else
  msg->AddFloat(MSGSTORE_NAME_SCALING,MSGSTORE_INTERNAL_SCALING_DEFAULT);
*/
}
void JobSetupWindow::SetPreview(bool value)
{
  bool temp;
  if (fmsg->FindBool(MSGSTORE_NAME_PREVIEW,&temp) == B_OK)
  fmsg->ReplaceBool(MSGSTORE_NAME_PREVIEW,value);
  else
  fmsg->AddBool(MSGSTORE_NAME_PREVIEW,value);
}
// event
void JobSetupWindow::OnEventAllPage()
{
  Lock();
  SetEnableSelPageField(false);
  Unlock();
}			
void JobSetupWindow::OnEventSelPage()
{
  Lock();
  SetEnableSelPageField(true);
  Unlock();
}
void JobSetupWindow::SetEnableSelPageField(bool value)
{
  tv_page_start->SetEnabled(value);
  tv_page_end->SetEnabled(value);
}

void JobSetupWindow::UpdateInkLevelStatus(const char *label)
{
  lb_ink_level->SetText(label);
}
void JobSetupWindow::UpdateFunctionAvailability()
{
  btn_nozzle_check->SetEnabled(faddon->SupportNozzleCheck());
  btn_head_clean->SetEnabled(faddon->SupportHeadClean());
  btn_change_cartridge->SetEnabled(faddon->SupportChangeCartridge());
  if (faddon->SupportInkLevel())
  {
    btn_ink_level->SetEnabled(true);
    UpdateInkLevelStatus("Status: Ready");
    faddon->InkInfo(fprinter_info,(InkLevelGroup *) inklevel);
  }
  else
  {
    btn_ink_level->SetEnabled(false);
    UpdateInkLevelStatus("Status: Not Available");
  }
}

void JobSetupWindow::OnEventCopyNum()
{
  int value;
  if ((sscanf(tf_copies->Text(),"%d",&value) < 0) || (value < 1))
  tf_copies->SetText("1");
}
void JobSetupWindow::OnEventCollateOrder()
{
  Lock();
  bv_copies->SetTo(chk_collate->Value(),chk_reverse->Value());
  Unlock();
}
void JobSetupWindow::OnEventReverseOrder()
{
  Lock();
  bv_copies->SetTo(chk_collate->Value(),chk_reverse->Value());
  Unlock();
}
void JobSetupWindow::OnEventPaperType()
{
  Lock();
  SetSDQualityFromGUI();
  Unlock();
}
void JobSetupWindow::OnEventColorMode()
{
  Lock();
  bv_color_mode->ColorPrinter(rb_mode_color->Value());
  SetSDQualityFromGUI();
  Unlock();
}
void JobSetupWindow::OnEventQualityScale()
{
  Lock();
  SetLBQualityFromGUI();  
  SetBVQualityFromGUI();
  Unlock();
}
void JobSetupWindow::SetLBQualityFromGUI()
{
  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  // set the quality slider to this quality mode 
  lb_resolution->SetTo(data[index[paper_type_index].offset + sd_quality->Value()].xres,data[index[paper_type_index].offset + sd_quality->Value()].yres);  
}
void JobSetupWindow::SetBVQualityFromGUI()
{
  bv_quality->Value(sd_quality->Value());
  bv_quality->Invalidate();
}
bool JobSetupWindow::IsGUIColorMode()
{
  return rb_mode_color->Value();
}
int32 JobSetupWindow::GetGUISelectPaperType()
{
  int32 paper_type_index = mn_paper_type->IndexOf(mn_paper_type->FindMarked());
  if (paper_type_index < 0)
  {
    mn_paper_type->ItemAt(0)->SetMarked(true);
    paper_type_index = 0;
  }
  return paper_type_index;
}
void JobSetupWindow::SetSDQualityFromGUI()
{
  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  // set the quality slider to this quality mode 
  CreateSDQuality(index[paper_type_index].relative_def,index[paper_type_index].count);
  bv_quality->Range(index[paper_type_index].count - 1);
  SetLBQualityFromGUI();
  SetBVQualityFromGUI();
}  
void JobSetupWindow::CreateSDQuality(int value,int count)
{
  if (sd_quality != NULL)
  {
    f_quality_box->RemoveChild(sd_quality);
    delete sd_quality;
  }
  sd_quality = new BSlider(f_quality_rect,"",JobSetupWindowSpace::LB_QUALITY,new BMessage(JobSetupWindowSpace::MSGCODE_QUALITY_SCALE),0,count - 1,B_TRIANGLE_THUMB);
  sd_quality->SetHashMarks(B_HASH_MARKS_BOTTOM);
  sd_quality->SetHashMarkCount(count);
  const rgb_color color = {0,0,255,0};
  sd_quality->UseFillColor(true,&color);
  sd_quality->SetLimitLabels(JobSetupWindowSpace::LB_MIN_QUALITY,JobSetupWindowSpace::LB_MAX_QUALITY);
  f_quality_box->AddChild(sd_quality);
  sd_quality->SetValue(value);
}