/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "PageSetupWindow.h"
PageSetupWindow::PageSetupWindow(PrinterAddon *addon,uint8 unit)
:BWindow(CenterOfScreen(PageSetupWindowSpace::DEF_WIDTH,PageSetupWindowSpace::DEF_HEIGHT),PageSetupWindowSpace::DEF_TITLE,B_TITLED_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_WILL_DRAW)
,faddon(addon),fsem(0),fmsg(NULL),funit(unit)
{
  BuildGUI();
  tb_view->Select(0);
  fsem = create_sem(0,"");
}
void PageSetupWindow::Quit()
{
  delete_sem(fsem); 
  BWindow::Quit();
}
void PageSetupWindow::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case PageSetupWindowSpace::MSGCODE_COLLATE_ORDER:
    case PageSetupWindowSpace::MSGCODE_REVERSE_ORDER:
    OnOrderChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_NUM_OF_COPIES:
    OnNumberOfCopiesChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_PORTRAIT:
    case PageSetupWindowSpace::MSGCODE_LANDSCAPE:
    case PageSetupWindowSpace::MSGCODE_STANDARD:
    case PageSetupWindowSpace::MSGCODE_CENTERED:
    OnStyleChange();
    break;
    
    case PageSetupWindowSpace::MSGCODE_DISCARD:
    OnDiscard();
    break;
    
    case PageSetupWindowSpace::MSGCODE_REVERT:
    OnRevert();
    break;

    case PageSetupWindowSpace::MSGCODE_SAVE:
    OnSave();
    break;
    
    case PageSetupWindowSpace::MSGCODE_PAPERSIZE:
    OnUserDefinePaper();
    break;
    
    case PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT:
    OnUserDefinePageDlgResult(msg);
    break;

    case PageSetupWindowSpace::MSGCODE_PAPERTYPE:
    OnEventPaperType();
    break;
    
    case PageSetupWindowSpace::MSGCODE_COLOR:
    OnEventColorMode();
    break;
    
    case PageSetupWindowSpace::MSGCODE_QUALITY_SCALE:
    OnEventQualityScale();
    break;
        
    default:
    BWindow::MessageReceived(msg);
  }
}
void PageSetupWindow::Go(BMessage *msg)
{
  fmsg = msg;
  // set value to what in the message
  Lock();
  SetTo(msg);
  Unlock();
  Show();
  // wait
  acquire_sem(fsem);
  Lock();
  Quit();
}
void PageSetupWindow::BuildGUI()
{
  BRect rect_bg_box,tb_sub_rect,tb_rect,r1,r2;
  BRect rect_sub_box;
  BBox  *sub_box;
  BRect bound = Bounds();
  bound.bottom++;
  bound.right++;  
  
  // adding mainbox 
  BBox *mainbox = new BBox(bound,NULL);
  mainbox->SetBorder(B_NO_BORDER);
  AddChild(mainbox);
  
  // create tab view
  tb_rect = BRect(5,5,bound.Width() - 5,bound.Height() - 51);
  tb_view = new BTabView(tb_rect,"tabview");
  mainbox->AddChild(tb_view);

  // adjusting tab view RECT  
  tb_sub_rect = tb_view->Bounds();
  tb_sub_rect.bottom -= tb_view->TabHeight() + 2;
  tb_sub_rect.right -= 2;

  // ===================================================  
  // create pape format tab view
  BTab* rtab = new BTab();
  BBox *bg_box = new BBox(tb_sub_rect);
  bg_box->SetBorder(B_NO_BORDER);
  rect_bg_box = bg_box->Bounds();  
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(PageSetupWindowSpace::LB_TAB_PAGEFORMAT);

  // make paper size menu
  int kleft = (int) be_plain_font->StringWidth(PageSetupWindowSpace::LB_PAPERFEED) + (int) be_plain_font->StringWidth(" ");
  mn_paper = new BPopUpMenu(PageSetupWindowSpace::LB_PAPERTYPE_EMPTY); 

  if (funit == PRINTER_UNIT_INCH)
  mn_userdefine_paper_item = new BMenuItem(g_support_page_info[0].name_inch,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));
  else
  mn_userdefine_paper_item = new BMenuItem(g_support_page_info[0].name_cm,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));

  mn_paper->AddItem(mn_userdefine_paper_item);
  mn_paper->AddSeparatorItem();

  BMenuItem *mn_paper_item;
  for (int i = 1;i < 21;i++)
  {
    if (funit == PRINTER_UNIT_INCH)
    mn_paper_item = new BMenuItem(g_support_page_info[i].name_inch,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));
    else
    mn_paper_item = new BMenuItem(g_support_page_info[i].name_cm,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERSIZE));

    mn_paper->AddItem(mn_paper_item);
  }
  BRect r = BRect(8,8,tb_sub_rect.Width() - 8,38);
  BMenuField *mf_paper = new BMenuField(r,NULL,PageSetupWindowSpace::LB_PAPERSIZE,mn_paper);
  mf_paper->SetDivider(kleft);
  mf_paper->SetAlignment(B_ALIGN_LEFT);
  bg_box->AddChild(mf_paper);
  
  r.top += 30;
  r.bottom += 30;
  
  // make paper feed menu
  mn_feed = new BPopUpMenu("paper feed");
  LoadPaperFeederList();
  BMenuField *mf_feed = new BMenuField(r,NULL,PageSetupWindowSpace::LB_PAPERFEED,mn_feed);
  mf_feed->SetDivider(kleft);
  mf_feed->SetAlignment(B_ALIGN_LEFT);
  bg_box->AddChild(mf_feed);
  
  // add printer view box
  r1 = BRect(8,r.bottom + 5,165,tb_sub_rect.Height() - 20);
  BBox *printer_box = new BBox(r1);
  bg_box->AddChild(printer_box);
  BRect bv_style_rect = printer_box->Bounds();
  bv_style_rect.top += 2;
  bv_style_rect.left += 2;
  bv_style_rect.right -= 2;
  bv_style_rect.bottom -= 2;
  bv_style = new PagePrintStyleView(bv_style_rect);
  printer_box->AddChild(bv_style);
  // Adject rect for COPIES
  r2 = BRect(r1.right + 10,r1.top - 10,tb_sub_rect.Width() - 8,r1.top + 75);
  BBox *tempbox;

  BRect tempbox_rect;
  
  // add copies group
  tempbox = new BBox(r2);
  tempbox_rect = tempbox->Bounds();
  tempbox->SetLabel(PageSetupWindowSpace::LB_COPIES_GROUP);
  bg_box->AddChild(tempbox);

  // adding copies bitmap view  
  BRect rect1 = BRect(10,20,tempbox_rect.Width() - 115,70);
  bv_copies = new PageOrderView(rect1);
  tempbox->AddChild(bv_copies);

  // adding textcontrol for # copies
  BRect rect2 = BRect(tempbox_rect.Width() - 104,17,tempbox_rect.Width() - 28,37);  
  tf_copies = new BTextControl(rect2,NULL,PageSetupWindowSpace::LB_COPIES,"1",new BMessage(PageSetupWindowSpace::MSGCODE_NUM_OF_COPIES));
  tf_copies->SetDivider(be_plain_font->StringWidth(PageSetupWindowSpace::LB_COPIES));
  tf_copies->TextView()->SetMaxBytes(2);
  for (uint32 c = 0;c < 255;c++)
  {
    if (c >= '0' && c <= '9')
    tf_copies->TextView()->AllowChar(c);
    else
    tf_copies->TextView()->DisallowChar(c);
  }  
  tempbox->AddChild(tf_copies);

  rect2.right += 20;  
  rect2.top += 23;
  rect2.bottom += 23;

  // adding collate copies option
  chk_collate = new BCheckBox(rect2,NULL,PageSetupWindowSpace::LB_COLLATE,new BMessage(PageSetupWindowSpace::MSGCODE_COLLATE_ORDER));
  tempbox->AddChild(chk_collate);

  rect2.top += 22;
  rect2.bottom += 22;
  
  // adding reverse order option
  chk_reverse = new BCheckBox(rect2,NULL,PageSetupWindowSpace::LB_REVERSE,new BMessage(PageSetupWindowSpace::MSGCODE_REVERSE_ORDER));
  tempbox->AddChild(chk_reverse);  
  
  // adding orientation group
  r2.top = r2.bottom + 4;
  r2.bottom = r2.top + 60;
  
  tempbox = new BBox(r2);
  tempbox->SetLabel(PageSetupWindowSpace::LB_ORIENTATION_GROUP);
  bg_box->AddChild(tempbox);

  // add portrait option
  BRect r3 = BRect(13,15,90,28); 
  rb_portrait = new BRadioButton(r3,NULL,PageSetupWindowSpace::LB_PORTRAIT,new BMessage(PageSetupWindowSpace::MSGCODE_PORTRAIT));
  tempbox->AddChild(rb_portrait);

  // add landscape option
  BRect r4 = BRect(tempbox_rect.Width() - 104,r3.top,tempbox_rect.Width() - 8,r3.bottom);
  rb_landscape = new BRadioButton(r4,NULL,PageSetupWindowSpace::LB_LANDSCAPE,new BMessage(PageSetupWindowSpace::MSGCODE_LANDSCAPE));
  tempbox->AddChild(rb_landscape);

  // add rotate 180 degree option
  BRect r5 = r3;
  
  r5.top += 20;
  r5.bottom += 20;
  r5.right = tempbox_rect.Width() - 10;
  chk_180rotate = new BCheckBox(r5,NULL,PageSetupWindowSpace::LB_180ROTATE,NULL);
  tempbox->AddChild(chk_180rotate);

  // add printable group
  r2.top = r2.bottom + 4;
  r2.bottom = r2.top + 66;
  tempbox = new BBox(r2);
  tempbox->SetLabel(PageSetupWindowSpace::LB_PRINTABLEAREA_GROUP);
  bg_box->AddChild(tempbox);
  
  r3.top += 5;
  r3.bottom += 5;
  rb_parea_standard = new BRadioButton(r3,NULL,PageSetupWindowSpace::LB_STANDARD,new BMessage(PageSetupWindowSpace::MSGCODE_STANDARD));
  tempbox->AddChild(rb_parea_standard);

  r4.top += 5;
  r4.bottom += 5;
  rb_parea_centered = new BRadioButton(r4,NULL,PageSetupWindowSpace::LB_CENTERED,new BMessage(PageSetupWindowSpace::MSGCODE_CENTERED));
  tempbox->AddChild(rb_parea_centered);
  
  // ===================================================  
  // create OPTION tab view
  rtab = new BTab();
  bg_box = new BBox(tb_sub_rect,"hello");
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(PageSetupWindowSpace::LB_TAB_OPTION);
  
  // Create paper type group 
  r1 = BRect(10,10,rect_bg_box.Width() - 10,80);
  sub_box = new BBox(r1);
  rect_sub_box = sub_box->Bounds();
  sub_box->SetLabel(PageSetupWindowSpace::LB_PAPERTYPE_GROUP);
  bg_box->AddChild(sub_box);

  // create paper type menu
  mn_paper_type = new BPopUpMenu("");
  BMenuField *mf_paper_type = new BMenuField(BRect(10,25,rect_sub_box.Width() - 10,50),NULL,"",mn_paper_type);
  mf_paper_type->SetDivider(0);
  mf_paper_type->SetAlignment(B_ALIGN_LEFT);
  sub_box->AddChild(mf_paper_type);  
  // load independence setting
  LoadPaperType();
  
  // Create paper type group 
  r1 = BRect(10,90,rect_bg_box.Width() - 10,160);
  sub_box = new BBox(r1);
  rect_sub_box = sub_box->Bounds();
  sub_box->SetLabel(PageSetupWindowSpace::LB_MODE_COLOR_GROUP);
  bg_box->AddChild(sub_box);

  r1.left = 10;
  r1.right = 10 + 32;
  r1.top  = (rect_sub_box.Height() - 32) / 2;
  r1.bottom = r1.top + 32;
    
  bv_color_mode = new PrintingModeView(r1,false);
  sub_box->AddChild(bv_color_mode);
  
  r2 = BRect(r1.right + 10,15,rect_sub_box.Width() - 10,40);
  rb_mode_color = new BRadioButton(r2,NULL,PageSetupWindowSpace::LB_MODE_COLOR_COLOR,new BMessage(PageSetupWindowSpace::MSGCODE_COLOR));
  sub_box->AddChild(rb_mode_color);
  r2.InsetBy(0,25);

  rb_mode_mono = new BRadioButton(r2,NULL,PageSetupWindowSpace::LB_MODE_COLOR_MONO,new BMessage(PageSetupWindowSpace::MSGCODE_COLOR));
  sub_box->AddChild(rb_mode_mono);
  
  r1 = BRect(10,170,rect_bg_box.Width() - 10,rect_bg_box.Height() - 10);
  f_quality_box = new BBox(r1);
  rect_sub_box = f_quality_box->Bounds();
  f_quality_box->SetLabel(PageSetupWindowSpace::LB_QUALITY_GROUP);
  bg_box->AddChild(f_quality_box);
  
  r1 = BRect(10,30,78,68 + 30);
  bv_quality = new PrintingQualityView(r1,2,0);
  f_quality_box->AddChild(bv_quality);
 
  r2 = f_quality_rect = BRect(r1.right + 40,20,r1.right + 120 + 40,65);
  sd_quality = NULL;
  
  // add resolution string view
  r2.top = r2.bottom;
  r2.bottom = r2.top + 20;
  r2.right = rect_sub_box.Width() - 10;
  lb_resolution = new ResolutionView(r2);
  lb_resolution->SetTo(360,360);
  f_quality_box->AddChild(lb_resolution);
  // ===================================================  
/*
  // create position tab view
  rtab = new BTab();
  bg_box = new BBox(tb_sub_rect,NULL);
  bg_box->SetBorder(B_NO_BORDER);
  tb_view->AddTab(bg_box,rtab);
  rtab->SetLabel(PageSetupWindowSpace::LB_TAB_POSITION);
*/
  // ===================================================  

  // Adding buttons 
  bound = mainbox->Bounds();
  BRect btn_rect = BRect(10,bound.Height() - 38,100,bound.Height() - 20);
  btn_discard = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_DISCARD,new BMessage(PageSetupWindowSpace::MSGCODE_DISCARD));
  mainbox->AddChild(btn_discard);

  btn_rect.left = bound.Width() - 200;
  btn_rect.right = btn_rect.left + 90;

  btn_revert = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_REVERT,new BMessage(PageSetupWindowSpace::MSGCODE_REVERT));
  btn_revert->SetEnabled(false);
  mainbox->AddChild(btn_revert);

  btn_rect.left = bound.Width() - 100;
  btn_rect.right = bound.Width() - 10;
  btn_save = new BButton(btn_rect,NULL,PageSetupWindowSpace::LB_SAVE,new BMessage(PageSetupWindowSpace::MSGCODE_SAVE));
  btn_save->MakeDefault(true);
  mainbox->AddChild(btn_save);
}

void PageSetupWindow::OnOrderChange()
{
  Lock();
  bv_copies->SetTo(chk_collate->Value(),chk_reverse->Value());
  Unlock();
}

void PageSetupWindow::OnStyleChange()
{
  Lock();
  bv_style->SetTo(rb_portrait->Value(),rb_parea_standard->Value());
  Unlock();
}

void PageSetupWindow::OnDiscard()
{
  Lock();
  if (fmsg != NULL)
  fmsg->what = ('baad');
  release_sem(fsem);
  Unlock();   
}
void PageSetupWindow::OnRevert()
{
  Lock();
  Unlock();   
}
void PageSetupWindow::OnSave()
{
  Lock();
  if (fmsg != NULL)
  {
    fmsg->what = (uint32) 'okok';  
    Retrieve(fmsg);
  }
  release_sem(fsem);
  Unlock();   
}
void PageSetupWindow::Retrieve(BMessage *msg)
{
  if (msg != NULL)
  {
    int32 int32_value;
    float float_value;
    bool bool_value;
    float user_define_width,user_define_height;
    int32 print_style;
    bool p_area_normal;
    BRect rect;
    int32 paper_size;
    BRect paper_rect;
    BRect print_rect;             
    // paper_feeder
    if (msg->FindInt32(MSGSTORE_NAME_PAPER_FEEDER,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_PAPER_FEEDER,GetPaperFeeder());
    else
    msg->AddInt32(MSGSTORE_NAME_PAPER_FEEDER,GetPaperFeeder());

    // # copies
    if (msg->FindInt32(MSGSTORE_NAME_COPIES,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
    else
    msg->AddInt32(MSGSTORE_NAME_COPIES,GetNumberOfCopies());
    // -------------------------------------------------------
    
    // Collate Order    
    if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
    else
    msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,chk_collate->Value());
    // -------------------------------------------------------

    // Reverse Order
    if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
    else
    msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,chk_reverse->Value());
    // -------------------------------------------------------
    
    // Rotate
    if (msg->FindBool(MSGSTORE_NAME_ROTATE_180,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_ROTATE_180,chk_180rotate->Value());
    else
    msg->AddBool(MSGSTORE_NAME_ROTATE_180,chk_180rotate->Value());
    // -------------------------------------------------------
 
  	/* ========================================================== */
  	// paper type
  	int32 paper_type_index;
  	paper_type_index = GetGUISelectPaperType();
  	if (msg->FindInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,&int32_value) == B_OK)
  	msg->ReplaceInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,paper_type_index);
  	else
  	msg->AddInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,paper_type_index);
  
  	// color mode
  	if (msg->FindBool(MSGSTORE_NAME_PRINTING_COLOR,&bool_value) == B_OK)
  	msg->ReplaceBool(MSGSTORE_NAME_PRINTING_COLOR,rb_mode_color->Value());
  	else
  	msg->AddBool(MSGSTORE_NAME_PRINTING_COLOR,rb_mode_color->Value());
  
  	// quality index
  	if (msg->FindInt32(MSGSTORE_NAME_QUALITY,&int32_value) == B_OK)
  	msg->ReplaceInt32(MSGSTORE_NAME_QUALITY,sd_quality->Value());
  	else
  	msg->AddInt32(MSGSTORE_NAME_QUALITY,sd_quality->Value());

    // scaling
    if (msg->FindFloat(MSGSTORE_NAME_INTERNAL_SCALE,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_INTERNAL_SCALE,MSGSTORE_INTERNAL_SCALING_DEFAULT);
    else
    msg->AddFloat(MSGSTORE_NAME_INTERNAL_SCALE,MSGSTORE_INTERNAL_SCALING_DEFAULT);
    // -------------------------------------------------------

    // paper_size
    paper_size = GetPaperSize();
    if (msg->FindInt32(MSGSTORE_NAME_PAPER_SIZE,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_PAPER_SIZE,paper_size);
    else
    msg->AddInt32(MSGSTORE_NAME_PAPER_SIZE,paper_size);

    // User Define Paper Width
    user_define_width = GetUserDefinePaperWidth();
    if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,user_define_width);
    else
    msg->AddFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,user_define_width);

    // User Define Paper Height
    user_define_height = GetUserDefinePaperHeight();
    if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,user_define_height);
    else
    msg->AddFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,user_define_height);
        
    // printable_style
    p_area_normal = rb_parea_standard->Value() == 1;
    if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&bool_value) == B_OK)
    msg->ReplaceBool(MSGSTORE_NAME_PRINTABLE_STYLE,p_area_normal);
    else
    msg->AddBool(MSGSTORE_NAME_PRINTABLE_STYLE,p_area_normal);
    
    // orientation
    if (rb_portrait->Value())
    print_style = MSGSTORE_ORIENTATION_PORTRAIT;
    else
    print_style = MSGSTORE_ORIENTATION_LANDSCAPE;
    if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&int32_value) == B_OK)
    msg->ReplaceInt32(MSGSTORE_NAME_ORIENTATION,print_style);
    else
    msg->AddInt32(MSGSTORE_NAME_ORIENTATION,print_style);
    // -------------------------------------------------------

    // paper_rect
/*
    BRect paper_rect = GetPrintRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
    BRect print_rect = paper_rect;
  
    paper_rect = BRect(0,0,paper_rect.Width(),paper_rect.Height());
    if (msg->FindRect(MSGSTORE_NAME_PAPER_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);
    else
    msg->AddRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);

    // printable_rect
    if (msg->FindRect(MSGSTORE_NAME_PRINTABLE_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);
    else
    msg->AddRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);
*/
    // paper rect
    paper_rect = GetPaperRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
    if (msg->FindRect(MSGSTORE_NAME_PAPER_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);
    else
    msg->AddRect(MSGSTORE_NAME_PAPER_RECT,paper_rect);
  
    // printable rect
    print_rect = GetPrintRect(funit,paper_size,user_define_width,user_define_height,p_area_normal,print_style == MSGSTORE_ORIENTATION_PORTRAIT);
    if (msg->FindRect(MSGSTORE_NAME_PRINTABLE_RECT,&rect) == B_OK)
    msg->ReplaceRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);
    else
    msg->AddRect(MSGSTORE_NAME_PRINTABLE_RECT,print_rect);

    // scaling 
    if (msg->FindFloat(MSGSTORE_NAME_SCALING,&float_value) == B_OK)
    msg->ReplaceFloat(MSGSTORE_NAME_SCALING,MSGSTORE_INTERNAL_SCALING_DEFAULT);
    else
    msg->AddFloat(MSGSTORE_NAME_SCALING,MSGSTORE_INTERNAL_SCALING_DEFAULT);
  }
}
int32 PageSetupWindow::GetPaperSize()			// return paper kind
{
  return fpaper_size;
}

float PageSetupWindow::GetUserDefinePaperWidth()	// return user define paper width
{
  return fuserdefine_width;
}
float PageSetupWindow::GetUserDefinePaperHeight()	// return user define paper height
{
  return fuserdefine_height;
}
/*
BRect PageSetupWindow::GetPaperRect()				// return paper rect in pixel
{
  int32 width,height;
  BRect rect;
  if (fpaper_size == PAGE_USER_DEFINE)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      width  = (int32) (fuserdefine_width * 360.0f);
      height = (int32) (fuserdefine_height * 360.0f);
    } 
    else
    {
      width  = (int32) (CmToInch(fuserdefine_width) * 360.0f);
      height = (int32) (CmToInch(fuserdefine_height) * 360.0f);
    }
  }
  else
  {
    width  = (int32) g_support_page_info[fpaper_size].max_inch_size.width * 360.0f;
    height = (int32) g_support_page_info[fpaper_size].max_inch_size.height * 360.0f; 
  }   

  rect.left   = 0;
  rect.right  = width;
  rect.top    = 0;
  rect.bottom = height;
  if (rb_landscape->Value())
  rect = BRect(rect.top,rect.left,rect.bottom,rect.right);
  return rect;
}
BRect PageSetupWindow::GetPrintableRect()			// return printable rect uh.. in pixel
{
  BRect rect;
  if (fpaper_size == PAGE_USER_DEFINE)
  {
    int32 width,height;
    if (funit == PRINTER_UNIT_INCH)
    {
      width  = (int32) (fuserdefine_width * 360.0f);
      height = (int32) (fuserdefine_height * 360.0f);
    } 
    else
    {
      width  = (int32) (CmToInch(fuserdefine_width) * 360.0f);
      height = (int32) (CmToInch(fuserdefine_height) * 360.0f);
    }

    rect.left   = 42.0f;
    rect.right  = width - 42.0f;
    rect.top    = 42.0f;
    rect.bottom = height - 354;
    if (rb_landscape->Value())
    rect = BRect(rect.top,rect.left,rect.bottom,rect.right);
  }
  else
  {
    if (rb_parea_standard->Value())
    {
      rect.left   = g_support_page_info[fpaper_size].printable_area_nornal.left;
      rect.right  = g_support_page_info[fpaper_size].printable_area_nornal.left +  g_support_page_info[fpaper_size].printable_area_nornal.width;
      rect.top    = g_support_page_info[fpaper_size].printable_area_nornal.top;
      rect.bottom = g_support_page_info[fpaper_size].printable_area_nornal.top + g_support_page_info[fpaper_size].printable_area_nornal.height;
    }
    else
    {
      rect.left   = g_support_page_info[fpaper_size].printable_area_center.left;
      rect.right  = g_support_page_info[fpaper_size].printable_area_center.left +  g_support_page_info[fpaper_size].printable_area_nornal.width;
      rect.top    = g_support_page_info[fpaper_size].printable_area_center.top;
      rect.bottom = g_support_page_info[fpaper_size].printable_area_center.top + g_support_page_info[fpaper_size].printable_area_nornal.height;
    }
  }
  return rect;
}
*/
int32 PageSetupWindow::GetNumberOfCopies()			// return number of copies
{
  int copies = 0;
  char temp[3];
  sprintf(temp,"%s",tf_copies->Text());
  if ((sscanf(temp,"%d",&copies) < 1) || (copies < 0))
  {
    copies = 1;
    tf_copies->SetText("1");
  }
  return copies;
}
void PageSetupWindow::UpdateUserDefinePageSizeMenuLabel()
{
  char temp[256];
  if (funit == PRINTER_UNIT_INCH)
  sprintf(temp,"%s %0.2fx%0.2f inch",g_support_page_info[PAGE_USER_DEFINE].name_inch,fuserdefine_width,fuserdefine_height);
  else
  sprintf(temp,"%s %0.2fx%0.2f cm",g_support_page_info[PAGE_USER_DEFINE].name_cm,fuserdefine_width,fuserdefine_height);
  mn_userdefine_paper_item->SetLabel(temp);  
}
void PageSetupWindow::ReselectPageSizeMenu()
{
  int32 index = fpaper_size;
  if (fpaper_size > PAGE_USER_DEFINE)
  index++;
  mn_paper->ItemAt(index)->SetMarked(true);
}

void PageSetupWindow::OnNumberOfCopiesChange()
{
  int copies;
  char temp[3];

  Lock();
  sprintf(temp,"%s",tf_copies->Text());
  if (sscanf(temp,"%d",&copies) < 1)
  tf_copies->SetText("1");
  else if (copies < 1)
  tf_copies->SetText("1");
  Unlock();
}
void PageSetupWindow::OnUserDefinePaper()
{
  uint32 select_index;
  Lock();
  // get select index
  BMenuItem *markeditem = mn_paper->FindMarked();
  if (markeditem != NULL)
  {
    int32 markedindex = mn_paper->IndexOf(markeditem);
    if (markedindex < 0)
    {
      fpaper_size = select_index = PAGE_A4;
      mn_paper->ItemAt(select_index + 1)->SetMarked(true);
      Unlock();
      return;
    }
    else if (markedindex > 0)
    select_index = markedindex - 1;
    else
    select_index = markedindex;
  }
  else
  {
    fpaper_size = select_index = PAGE_A4;
    mn_paper->ItemAt(select_index + 1)->SetMarked(true);
    Unlock();
    return;
  }

  if (select_index == PAGE_USER_DEFINE)
  {
    if (funit == PRINTER_UNIT_INCH)
    userdlg = new UserDefinePageDlg(funit,
    								ScreenToInch(faddon->MinPageWidth()),
    								ScreenToInch(faddon->MinPageHeight()),
    								ScreenToInch(faddon->MaxPageWidth()),
    								ScreenToInch(faddon->MaxPageHeight()),
  								  	new BMessage(PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT),this);
    else
    userdlg = new UserDefinePageDlg(funit,
    								ScreenToCm(faddon->MinPageWidth()),
    								ScreenToCm(faddon->MinPageHeight()),
    								ScreenToCm(faddon->MaxPageWidth()),
    								ScreenToCm(faddon->MaxPageHeight()),
  								  	new BMessage(PageSetupWindowSpace::MSGCODE_USERDEFINEPAGEDLG_RESULT),this);
    userdlg->SetTo(fuserdefine_width,fuserdefine_height);
    userdlg->Show();
  }
  else
  {
    fpaper_size = select_index;
  }  
  Unlock();
}
void PageSetupWindow::OnUserDefinePageDlgResult(BMessage *msg)
{
  int32 result;
  Lock();
  // get current select item
  msg->FindInt32("result",&result);
  if (result == B_OK)
  {
    msg->FindFloat("width",&fuserdefine_width);
    msg->FindFloat("height",&fuserdefine_height);
    // update menu item label
    UpdateUserDefinePageSizeMenuLabel();
    // save last page selection to user define
    fpaper_size = PAGE_USER_DEFINE;
  }
  // re-select the last selection
  ReselectPageSizeMenu();
  Unlock();
}
void PageSetupWindow::SetTo(BMessage *msg)
{
  BString str;
  if (msg == NULL)
  return;
  // Paper Type
  if (msg->FindInt32(MSGSTORE_NAME_PAPER_SIZE,&fpaper_size) != B_OK)
  fpaper_size = PAGE_A4;
  // User Define Width
  if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_WIDTH,&fuserdefine_width) == B_OK)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      if (fuserdefine_width > ScreenToInch(faddon->MaxPageWidth()))
      fuserdefine_width = ScreenToInch(faddon->MaxPageWidth());
      else if (fuserdefine_width < ScreenToInch(faddon->MinPageWidth()))
      fuserdefine_width = ScreenToInch(faddon->MinPageWidth());
    }
    else
    {
      if (fuserdefine_width > ScreenToCm(faddon->MaxPageWidth()))
      fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
      else if (fuserdefine_width < ScreenToCm(faddon->MinPageWidth()))
      fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
    }
  }
  else
  {
    if (funit == PRINTER_UNIT_INCH)
    fuserdefine_width = ScreenToInch(faddon->MinPageWidth());
    else
    fuserdefine_width = ScreenToCm(faddon->MinPageWidth());
  }
  // User Define Height
  if (msg->FindFloat(MSGSTORE_NAME_USERDEFINE_PAPER_HEIGHT,&fuserdefine_height) == B_OK)
  {
    if (funit == PRINTER_UNIT_INCH)
    {
      if (fuserdefine_height > ScreenToInch(faddon->MaxPageHeight()))
      fuserdefine_height = ScreenToInch(faddon->MaxPageHeight());
      else if (fuserdefine_height < ScreenToInch(faddon->MinPageHeight()))
      fuserdefine_height = ScreenToInch(faddon->MinPageHeight());
    }
    else
    {
      if (fuserdefine_height > ScreenToCm(faddon->MaxPageHeight()))
      fuserdefine_height = ScreenToCm(faddon->MaxPageHeight());
      else if (fuserdefine_height < ScreenToCm(faddon->MinPageHeight()))
      fuserdefine_height = ScreenToCm(faddon->MinPageHeight());
    }
  }
  else
  {
    if (funit == PRINTER_UNIT_INCH)
    fuserdefine_height = ScreenToInch(faddon->MinPageHeight());
    else
    fuserdefine_height = ScreenToCm(faddon->MinPageHeight());
  }  
  UpdateUserDefinePageSizeMenuLabel();
  ReselectPageSizeMenu();
  // -------------------------------------------------------

  int32 p_feeder;
  if (msg->FindInt32(MSGSTORE_NAME_PAPER_FEEDER,&p_feeder) == B_OK)
  {
    if (p_feeder >= mn_feed->CountItems())
    p_feeder = 0;
    if (mn_feed->CountItems() > 0)
    mn_feed->ItemAt(p_feeder)->SetMarked(true); 
  }
  
  // # copies
  int32 p_copies;
  if (msg->FindInt32(MSGSTORE_NAME_COPIES,&p_copies) == B_OK)
  {
    if (p_copies < 0)
    p_copies = 1;
    else if (p_copies > 99)
    p_copies = 99;
  }
  else
  p_copies = 1;

  BString str_copies;
  str_copies << (int32) p_copies;
  tf_copies->SetText(str_copies.String());
  // -------------------------------------------------------
    
  // Collate Order
  bool p_collate_order;
  if (msg->FindBool(MSGSTORE_NAME_COLLATE_ORDER,&p_collate_order) != B_OK)
  {
    p_collate_order = false;
    msg->AddBool(MSGSTORE_NAME_COLLATE_ORDER,p_collate_order);
  }
  chk_collate->SetValue(p_collate_order);
  // -------------------------------------------------------

  // Reverse Order
  bool p_reverse_order;
  if (msg->FindBool(MSGSTORE_NAME_REVERSE_ORDER,&p_reverse_order) != B_OK)
  {
    p_reverse_order = false;
    msg->AddBool(MSGSTORE_NAME_REVERSE_ORDER,p_reverse_order);
  }
  chk_reverse->SetValue(p_reverse_order);
  // -------------------------------------------------------
    
  // Rotate
  bool p_rotate_order;
  if (msg->FindBool(MSGSTORE_NAME_ROTATE_180,&p_rotate_order) != B_OK)
  {
    p_rotate_order = false;
    msg->AddBool(MSGSTORE_NAME_ROTATE_180,p_rotate_order);
  }
  chk_180rotate->SetValue(p_rotate_order);
  // -------------------------------------------------------

  bv_copies->SetTo(p_collate_order,p_reverse_order);

  // orientation
  int32 p_orientation;
  if (msg->FindInt32(MSGSTORE_NAME_ORIENTATION,&p_orientation) == B_OK)
  {
    if ((p_orientation != MSGSTORE_ORIENTATION_PORTRAIT) && (p_orientation != MSGSTORE_ORIENTATION_LANDSCAPE))
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
  }
  else
  {
    p_orientation = MSGSTORE_ORIENTATION_PORTRAIT;
    msg->AddInt32(MSGSTORE_NAME_ORIENTATION,p_orientation);
  }
  
  if (p_orientation == MSGSTORE_ORIENTATION_PORTRAIT)
  {
    rb_landscape->SetValue(false);
    rb_portrait->SetValue(true);
  }
  else
  {
    rb_portrait->SetValue(false);
    rb_landscape->SetValue(true);
  }
  // -------------------------------------------------------
  
  // printable area style
  bool normal_printable_area;
  if (msg->FindBool(MSGSTORE_NAME_PRINTABLE_STYLE,&normal_printable_area) != B_OK)
  normal_printable_area = true;
  
  if (normal_printable_area)
  {
    rb_parea_centered->SetValue(false);
    rb_parea_standard->SetValue(true);
  }
  else
  {
    rb_parea_standard->SetValue(false);
    rb_parea_centered->SetValue(true);
  }
  
  bv_style->SetTo(p_orientation == MSGSTORE_ORIENTATION_PORTRAIT,normal_printable_area);
  
  
  /* ==================================================================== */
  // Paper Kind
  int32 p_paper_media = 0;
  msg->FindInt32(MSGSTORE_NAME_QUALITY_PAPER_KIND,&p_paper_media);
  mn_paper_type->ItemAt(p_paper_media)->SetMarked(true);  
  /* ==================================================================== */
  
  /* ==================================================================== */
  bool p_color_printing = true;
  if (msg->FindBool(MSGSTORE_NAME_PRINTING_COLOR,&p_color_printing) < B_OK)
  p_color_printing = true;
  if (!faddon->SupportColorPrint())
  {
    p_color_printing = false;
    rb_mode_color->SetEnabled(false);
  }
  
  if (p_color_printing)
  rb_mode_color->SetValue(true);
  else
  rb_mode_mono->SetValue(true);
  bv_color_mode->ColorPrinter(p_color_printing);

  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  
  // Quality Index
  int32 quality_index;
  if (msg->FindInt32(MSGSTORE_NAME_QUALITY,&quality_index) < B_OK)
  quality_index = -1;
   
  // set the quality slider to this quality mode 
  if ((quality_index < 0) || (quality_index >= index[paper_type_index].count))
  {
    CreateSDQuality(index[paper_type_index].relative_def,index[paper_type_index].count);
    bv_quality->Range(index[paper_type_index].count - 1);
    SetLBQualityFromGUI();
    SetBVQualityFromGUI();
  }
  else
  {
    CreateSDQuality(quality_index,index[paper_type_index].count);
    bv_quality->Range(index[paper_type_index].count - 1);
    SetLBQualityFromGUI();
    SetBVQualityFromGUI();
  }
  /* ==================================================================== */  
  // scaling not yet implemented
}
void PageSetupWindow::LoadPaperFeederList()
{
  paper_feeder_st *data = faddon->PaperFeeder();
  while ((data != NULL) && (data->id >= 0))
  {
    BMenuItem *mn_paper_feed_item = new BMenuItem(data->name,NULL);
    mn_feed->AddItem(mn_paper_feed_item);
    data++;
  };
  
  if (mn_feed->CountItems() > 0)
  mn_feed->ItemAt(0)->SetMarked(true);
}
int32 PageSetupWindow::GetPaperFeeder()
{
  BMenuItem *item;
  if (mn_feed->CountItems() < 1)
  return -1;
  item = mn_feed->FindMarked();
  if (item == NULL)
  {
    item = mn_feed->ItemAt(0);
    item->SetMarked(true);
  }
  return mn_feed->IndexOf(item);
}
void PageSetupWindow::OnEventPaperType()
{
  Lock();
  SetSDQualityFromGUI();
  Unlock();
}
void PageSetupWindow::OnEventColorMode()
{
  Lock();
  bv_color_mode->ColorPrinter(rb_mode_color->Value());
  SetSDQualityFromGUI();
  Unlock();
}
void PageSetupWindow::OnEventQualityScale()
{
  Lock();
  SetLBQualityFromGUI();  
  SetBVQualityFromGUI();
  Unlock();
}

void PageSetupWindow::LoadPaperType()
{
  BMenuItem *newitem;
  support_paper_media_st *temp_paper_media;
  if ((temp_paper_media = faddon->SupportPaperMedia()) != NULL)
  {
    while (temp_paper_media->id >= 0)
    {
      newitem = new BMenuItem(temp_paper_media->name,new BMessage(PageSetupWindowSpace::MSGCODE_PAPERTYPE));
      mn_paper_type->AddItem(newitem);  
      temp_paper_media++;
    }
  }
  if (mn_paper_type->CountItems() == 0)
  {
    newitem = new BMenuItem("Plain paper",new BMessage(PageSetupWindowSpace::MSGCODE_PAPERTYPE));
    mn_paper_type->AddItem(newitem);  
    temp_paper_media++;
  }
}
bool PageSetupWindow::IsGUIColorMode()
{
  return rb_mode_color->Value();
}
int32 PageSetupWindow::GetGUISelectPaperType()
{
  int32 paper_type_index = mn_paper_type->IndexOf(mn_paper_type->FindMarked());
  if (paper_type_index < 0)
  {
    mn_paper_type->ItemAt(0)->SetMarked(true);
    paper_type_index = 0;
  }
  return paper_type_index;
}
void PageSetupWindow::SetSDQualityFromGUI()
{
  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  // set the quality slider to this quality mode 
  CreateSDQuality(index[paper_type_index].relative_def,index[paper_type_index].count);
  bv_quality->Range(index[paper_type_index].count - 1);
  SetLBQualityFromGUI();
  SetBVQualityFromGUI();
}
void PageSetupWindow::CreateSDQuality(int value,int count)
{
  if (sd_quality != NULL)
  {
    f_quality_box->RemoveChild(sd_quality);
    delete sd_quality;
  }
  sd_quality = new BSlider(f_quality_rect,"",PageSetupWindowSpace::LB_QUALITY,new BMessage(PageSetupWindowSpace::MSGCODE_QUALITY_SCALE),0,count - 1,B_TRIANGLE_THUMB);
  sd_quality->SetHashMarks(B_HASH_MARKS_BOTTOM);
  sd_quality->SetHashMarkCount(count);
  const rgb_color color = {0,0,255,0};
  sd_quality->UseFillColor(true,&color);
  sd_quality->SetLimitLabels(PageSetupWindowSpace::LB_MIN_QUALITY,PageSetupWindowSpace::LB_MAX_QUALITY);
  f_quality_box->AddChild(sd_quality);
  sd_quality->SetValue(value);
}
void PageSetupWindow::SetLBQualityFromGUI()
{
  int32 paper_type_index;
  print_quality_st *data;
  print_quality_index_st *index;
  
  if (IsGUIColorMode())
  {
    data  = faddon->SupportQualityColor();
    index = faddon->SupportQualityIndexColor(); 
  }
  else
  {
    data  = faddon->SupportQualityMono();
    index = faddon->SupportQualityIndexMono();
  }
  paper_type_index = GetGUISelectPaperType();
  // set the quality slider to this quality mode 
  lb_resolution->SetTo(data[index[paper_type_index].offset + sd_quality->Value()].xres,data[index[paper_type_index].offset + sd_quality->Value()].yres);  
}
void PageSetupWindow::SetBVQualityFromGUI()
{
  bv_quality->Value(sd_quality->Value());
  bv_quality->Invalidate();
}
