/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "PreviewView.h"
PreviewView::PreviewView(BRect rect,BBitmap *bitmap,float zoomscale)
:BView(rect,"Preview View", B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS)
,fbitmap(bitmap),fzoomscale(zoomscale)
{

}
void PreviewView::AttachedToWindow(void)
{
  FixupScrollbars();
  SetViewColor(B_TRANSPARENT_32_BIT);
}
void PreviewView::Draw(BRect draw_rect)
{
  SetHighColor(128,128,128);
  FillRect(draw_rect);
  BRect bitmap_bounds,view_bounds;
  BPoint draw_point;
  if (fbitmap != NULL)
  {
    view_bounds = Bounds();
    bitmap_bounds = fbitmap->Bounds();
    bitmap_bounds.right  *= fzoomscale;
    bitmap_bounds.bottom *= fzoomscale;
    if (bitmap_bounds.right > view_bounds.right)
    draw_point.x = 0;
    else
    draw_point.x = (view_bounds.right - bitmap_bounds.right) / 2;

    if (bitmap_bounds.bottom > view_bounds.bottom)
    draw_point.y = 0;
    else
    draw_point.y = (view_bounds.bottom - bitmap_bounds.bottom) / 2;
    bitmap_bounds.OffsetTo(draw_point);    
    DrawBitmap(fbitmap,bitmap_bounds);
  }
}
void PreviewView::FrameResized(float width,float height)
{
  FixupScrollbars();
  Invalidate();
}
void PreviewView::GetMaxSize(float *maxx, float *maxy)
{
  BWindow *mywin = Window();
  float mx, my;
  mx = my = 0.0f;
  if(mywin && mywin->Lock())
  {
    BRect imagebounds = fbitmap->Bounds();
    mx = floor((imagebounds.right)*fzoomscale);
    my = floor((imagebounds.bottom)*fzoomscale);
    mywin->Unlock();
  }
  if(maxx) 
  *maxx = mx;
  if(maxy) 
  *maxy = my;
}
void PreviewView::FixupScrollbars(void)
{
  BRect bound;
  BScrollBar *bar;
  bound = Bounds();
  float myPixelWidth = bound.Width();
  float myPixelHeight = bound.Height();
  float maxWidth = 1,maxHeight = 1;

  if(fbitmap != NULL)
  GetMaxSize(&maxWidth, &maxHeight);
		
  float propW = myPixelWidth/maxWidth;
  float propH = myPixelHeight/maxHeight;
	
  float rangeW = maxWidth - myPixelWidth;
  float rangeH = maxHeight - myPixelHeight;
  if (rangeW < 0) 
  rangeW = 0;
  if (rangeH < 0)
  rangeH = 0;


  if ((bar=ScrollBar(B_HORIZONTAL))!=NULL) 
  {
	bar->SetProportion(propW);
	bar->SetRange(0,rangeW);
	// Steps are 1/8 visible window for small steps
	//   and 1/2 visible window for large steps
	bar->SetSteps(myPixelWidth / 8.0, myPixelWidth / 2.0);
  } 

  if ((bar=ScrollBar(B_VERTICAL))!=NULL) 
  {
	bar->SetProportion(propH);
	bar->SetRange(0,rangeH);
	// Steps are 1/8 visible window for small steps
	//   and 1/2 visible window for large steps
	bar->SetSteps(myPixelHeight / 8.0, myPixelHeight / 2.0);
  }  
}
void PreviewView::SetZoomScale(float zoomscale)
{
  fzoomscale = zoomscale; 
}
float PreviewView::ZoomScale()
{
  return fzoomscale;
}
void PreviewView::SetBitmap(BBitmap *bitmap)
{
  fbitmap = bitmap;
}
BBitmap *PreviewView::Bitmap()
{
  return fbitmap;
}
void PreviewView::Update()
{
  FixupScrollbars();
  Invalidate();
}