/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "PreviewWindow.h"
PreviewWindow::PreviewWindow(SpoolFile *spool_file)
:BWindow(BRect(0,0,PreviewWindowSpace::DefWidth,PreviewWindowSpace::DefHeight),PreviewWindowSpace::WindowTitle,B_DOCUMENT_WINDOW,B_CURRENT_WORKSPACE)
,fsem(-1),fresult(false),fspool_file(spool_file),fpreview_bitmap(NULL),fpreviewview(NULL)
{
  // create semaphore
  fsem = create_sem(0,PreviewWindowSpace::WindowTitle);
  fpreview_bitmap = new PreviewSpoolBitmap(fspool_file);
  BuildGUI();
}
bool PreviewWindow::Go(bool inch_mode)
{
  float doc_width;
  float doc_height;
  Lock();

  // Read Document config
  ReadDocumentConfig();

  // Read Page format
  if (fspool_file->Portrait())
  {
    doc_width  = fspool_file->PaperRect().Width() * 5;
    doc_height = fspool_file->PaperRect().Height() * 5;
  }
  else
  {
    doc_width  = fspool_file->PaperRect().Height() * 5;
    doc_height = fspool_file->PaperRect().Width() * 5;
  } 

  UpdatePageFormat(doc_width,doc_height,inch_mode);

  fpage_current = 1;

  UpdateNavButtons();
  UpdatePageNumLabel();

  SetZoomSliderValue(1.0f);							
  SetZoomTextValue(1.0f);
  SetZoomMenuValue(1.0f);

  ReadPage();
  fpreviewview->Update();

  Show();
  Unlock();
  
  acquire_sem(fsem);

  Lock();
  Uninit();
  Quit();
  return fresult;  
}
void PreviewWindow::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case PreviewWindowSpace::MSGCODE_BTN_PRINT:
    OnPrintBtn();
    break;
    
    case PreviewWindowSpace::MSGCODE_BTN_CANCEL:
    OnCancelBtn();
    break;
    
    case PreviewWindowSpace::MSGCODE_BTN_NEXT:
    OnNextBtn();
    break;
    
    case PreviewWindowSpace::MSGCODE_BTN_PREV:
    OnPrevBtn();
    break;
    
    case PreviewWindowSpace::MSGCODE_SLIDER_ZOOM:
    OnSliderZoom();
    break;
    
    case PreviewWindowSpace::MSGCODE_TEXT_ZOOM:
    OnTextZoom();
    break;
    
    case PreviewWindowSpace::MSGCODE_MENU_ZOOM:
    OnMenuZoom();
    break;
    
    default:
    BWindow::MessageReceived(msg);
  }
}
bool PreviewWindow::QuitRequested(void)
{
  if (fsem >= B_OK)
  {
    release_sem(fsem);
    delete_sem(fsem);
  }
  return true;
}
void PreviewWindow::BuildGUI()
{
  /* ================================================================ */
  /* Re-Arrange window size and positon                               */
  // move window position to 0,0 + title_height
  int32 title_height = 25;
  MoveTo(5,title_height);
  // get display mode info
  display_mode display_data;
  GetScreenInfo(&display_data);
  int32 max_height = display_data.virtual_height - (title_height + 5);  
  int32 max_width  = display_data.virtual_width - 10;
  // resize window to max_width and max_height;
  ResizeTo(PreviewWindowSpace::DefWidth,max_height);
  // set maximun_width and maximun_height
  SetSizeLimits(PreviewWindowSpace::DefWidth,max_width,PreviewWindowSpace::DefHeight,max_height);   
  /* ================================================================ */
  
  BBox *panelbox;
  BBox *buttonbox;
  BRect panelbound;
  BRect buttonbound;
  BRect r1,r2;
  /* ================================================================ */
  /* Create Panel                                                     */
  panelbound = Bounds();
  panelbound.right += 2;
  panelbound.bottom = panelbound.top + PreviewWindowSpace::TopPanelHeight;
  // create panel view box
  panelbox = new BBox(panelbound,NULL,B_FOLLOW_LEFT_RIGHT);
  panelbox->SetBorder(B_NO_BORDER);
  // add a panel box to the window
  AddChild(panelbox);
  
  // Create Button Box
  buttonbound = BRect(8,8,PreviewWindowSpace::ButtonPanelWidth - 8,PreviewWindowSpace::TopPanelHeight - 8);
  buttonbox = new BBox(buttonbound,NULL);
  // Add Button Box
  panelbox->AddChild(buttonbox);
  /* ================================================================ */

  /* ================================================================ */  
  // Create PrintButton 
  r1 = BRect(8,8,70 + 8,28);
  fprintbtn  = new BButton(r1,PreviewWindowSpace::Label_Button_Print,PreviewWindowSpace::Label_Button_Print,new BMessage(PreviewWindowSpace::MSGCODE_BTN_PRINT));
  buttonbox->AddChild(fprintbtn);  
  r1.OffsetBy(74,0);
  fcancelbtn  = new BButton(r1,PreviewWindowSpace::Label_Button_Cancel,PreviewWindowSpace::Label_Button_Cancel,new BMessage(PreviewWindowSpace::MSGCODE_BTN_CANCEL));
  buttonbox->AddChild(fcancelbtn);  

  
  r2 = BRect(8,8 + 28,70 + 8,28 + 28);
  fprevbtn = new BButton(r2,PreviewWindowSpace::Label_Button_Prev,PreviewWindowSpace::Label_Button_Prev,new BMessage(PreviewWindowSpace::MSGCODE_BTN_PREV));
  buttonbox->AddChild(fprevbtn);
  r2.OffsetBy(74,0);
  fnextbtn = new BButton(r2,PreviewWindowSpace::Label_Button_Next,PreviewWindowSpace::Label_Button_Next,new BMessage(PreviewWindowSpace::MSGCODE_BTN_NEXT));
  buttonbox->AddChild(fnextbtn);  

  // Create Zoom scaling 
  r1.top = 8;
  r1.left = r1.right + 10;
  r1.right = r1.left + 100;
  fzoomslider = new BSlider(r1,"",PreviewWindowSpace::Label_Scale_Zoom,new BMessage(PreviewWindowSpace::MSGCODE_SLIDER_ZOOM),PreviewWindowSpace::ZOOM_MIN_VALUE,PreviewWindowSpace::ZOOM_MAX_VALUE);
  const rgb_color color = {0,0,255,0};
  fzoomslider->UseFillColor(true,&color);
  BString minstr;
  BString maxstr;
  minstr << PreviewWindowSpace::ZOOM_MIN_VALUE << "%";
  maxstr << PreviewWindowSpace::ZOOM_MAX_VALUE << "%";
  fzoomslider->SetLimitLabels(minstr.String(),maxstr.String());
  buttonbox->AddChild(fzoomslider);

  // Create Zoom Text
  r1.top = 8 + 10 + 7;
  r1.bottom = r1.top + 28;
  r1.left = r1.right + 5;
  r1.right = r1.left + 50;
  r2 = r1;
  fzoomtext = new BTextControl(r1,NULL,NULL,NULL,new BMessage(PreviewWindowSpace::MSGCODE_TEXT_ZOOM));
  for (uint32 c = 0;c < 255;c++)
  {
    if ((c >= '0' && c <= '9') || (c == '.'))
    fzoomtext->TextView()->AllowChar(c);
    else
    fzoomtext->TextView()->DisallowChar(c);
  }  
  buttonbox->AddChild(fzoomtext);

  // Create "%" String View
  r1.top -= 12;
  r1.bottom -= 12;
  r1.left = r1.right + 5;
  r1.right = r1.left + be_plain_font->StringWidth("% ");
  buttonbox->AddChild(new BStringView(r1,"%","%"));

  r2.left = r1.right + 5;
  r2.right = r2.left + be_plain_font->StringWidth(PreviewWindowSpace::Label_Select_Zoom) + 25;
  
  fmn_zoom = new BPopUpMenu(PreviewWindowSpace::Label_Select_Zoom);
  fmn_zoom->SetTriggersEnabled(false);
 
  BMenuItem *item;
  BString str;
  for (int i = 0;i < PreviewWindowSpace::Zoom_List_Total;i++)
  {
    str = "";
    str << (int32) PreviewWindowSpace::Zoom_List[i] << "%";   
    item = new BMenuItem(str.String(),new BMessage(PreviewWindowSpace::MSGCODE_MENU_ZOOM));
    fmn_zoom->AddItem(item);
  }
  fmn_zoom->InvalidateLayout();
  r2.top -= 2;
  r2.bottom -= 2;
  BMenuField *mfield = new BMenuField(r2,NULL,"",fmn_zoom);
  mfield->SetDivider(0);
  mfield->SetAlignment(B_ALIGN_LEFT);
  buttonbox->AddChild(mfield);  
  
  int32 lb_left = (int32) r2.right + 5;
  int32 lb_width = (int32) buttonbox->Bounds().Width() - (lb_left + 10);
  //be_plain_font->StringWidth(PreviewWindowSpace::Label_Page_Format) + 10; 
  r1 = BRect(lb_left,10,lb_left + lb_width,28);
  r2 = BRect(lb_left,35,lb_left + lb_width,53);
  // create "Page format Label";
  fpageformat = new BStringView(r1,PreviewWindowSpace::Label_Page_Format,PreviewWindowSpace::Label_Page_Format);
  buttonbox->AddChild(fpageformat);
  // create "Page size Label";
  fpagenumber = new BStringView(r2,PreviewWindowSpace::Label_Page_Number,PreviewWindowSpace::Label_Page_Number); 
  buttonbox->AddChild(fpagenumber);
  /* ================================================================ */  

  BRect preview_rect;
  preview_rect = Bounds();
  preview_rect.top += PreviewWindowSpace::TopPanelHeight + 1;
  preview_rect.right -= B_V_SCROLL_BAR_WIDTH;
  preview_rect.bottom -= B_H_SCROLL_BAR_HEIGHT;
  fpreviewview = new PreviewView(preview_rect,fpreview_bitmap->Bitmap(),1.0f);
  
  AddChild(new BScrollView("ImageScroll", fpreviewview, B_FOLLOW_ALL_SIDES, B_WILL_DRAW, true, true));

}
void PreviewWindow::OnPrintBtn()
{
  Lock();
  fresult = true;
  Unlock();
  release_sem(fsem);  
}
void PreviewWindow::OnCancelBtn()
{
  Lock();
  Unlock();
  release_sem(fsem);  
}
void PreviewWindow::OnNextBtn()
{
  Lock();
  fpage_current++;
  ReadPage();
  fpreviewview->Update();
  UpdateNavButtons();
  UpdatePageNumLabel();
  Unlock();
}    
void PreviewWindow::OnPrevBtn()
{
  Lock();
  fpage_current--;
  ReadPage();
  fpreviewview->Update();
  UpdateNavButtons();
  UpdatePageNumLabel();
  Unlock();
}    
void PreviewWindow::OnSliderZoom()
{
  Lock();
  float zoom_value = (fzoomslider->Value()) / 100.0f;
  fpreviewview->SetZoomScale(zoom_value);
  fpreviewview->Update();
  SetZoomMenuValue(zoom_value);
  SetZoomTextValue(zoom_value);
  Unlock();
}    
void PreviewWindow::OnTextZoom()
{
  float zoom_value;
  Lock();
  if (sscanf(fzoomtext->Text(),"%f",&zoom_value) == 1)
  {
    if (zoom_value > PreviewWindowSpace::ZOOM_MAX_VALUE)
    zoom_value = PreviewWindowSpace::ZOOM_MAX_VALUE;
    else if (zoom_value < PreviewWindowSpace::ZOOM_MIN_VALUE)
    zoom_value = PreviewWindowSpace::ZOOM_MIN_VALUE;
    
    zoom_value /= 100.0f;
    fpreviewview->SetZoomScale(zoom_value);
    fpreviewview->Update();
    SetZoomTextValue(zoom_value);
    SetZoomMenuValue(zoom_value);
    SetZoomSliderValue(zoom_value);
  }
  Unlock();
}    
void PreviewWindow::OnMenuZoom()
{
  Lock();
  // get zoom value
  int i = 0;
  int marked = -1;
  while (i < PreviewWindowSpace::Zoom_List_Total)
  {
    if (fmn_zoom->ItemAt(i)->IsMarked())
    {
      marked = i;
      break;
    }
    i++;
  }
  if (marked >= 0)
  {
    float zoom_value = PreviewWindowSpace::Zoom_List[marked] / 100.0f;
    fpreviewview->SetZoomScale(zoom_value);
    fpreviewview->Update();
    SetZoomTextValue(zoom_value);
    SetZoomSliderValue(zoom_value);
  }  
  Unlock();
}    
void PreviewWindow::Uninit()
{
  if (fpreview_bitmap != NULL)
  delete fpreview_bitmap;
}
void PreviewWindow::ReadDocumentConfig()
{
  // dump info
  fpage_total = fspool_file->PrintFileHeader().page_count;
}
void PreviewWindow::ReadPage()
{
  fspool_file->ReadPreviewPage(fpage_current,fpreview_bitmap);
}
void PreviewWindow::UpdateNavButtons()
{
  if (fpage_total > 0)
  {
    fnextbtn->SetEnabled(fpage_current < fpage_total);
    fprevbtn->SetEnabled(fpage_current > 1);
  }
  else
  {
    fnextbtn->SetEnabled(false);
    fprevbtn->SetEnabled(false);
  }
}
void PreviewWindow::UpdatePageNumLabel()
{
  BString str;
  if (fpage_total > 0)
  str << PreviewWindowSpace::Label_Page_Number << " " << fpage_current << " of " << fpage_total;
  else
  str << PreviewWindowSpace::Label_Page_Number << " " << "0 of 0"; 
  fpagenumber->SetText(str.String());
}
void PreviewWindow::UpdatePageFormat(float width,float height,bool inch_meansure)
{
  char str[70];
  if (inch_meansure)
  {
    width  = ScreenToInch((int32)width);
    height = ScreenToInch((int32)height); 
    sprintf(str,"%s %0.2f x %0.2f inch",PreviewWindowSpace::Label_Page_Format,width,height);
  }
  else
  {
    width  = ScreenToCm((int32)width);
    height = ScreenToCm((int32)height); 
    sprintf(str,"%s %0.2f x %0.2f cm",PreviewWindowSpace::Label_Page_Format,width,height);
  }
  fpageformat->SetText(str);
}
void PreviewWindow::SetZoomSliderValue(float v)
{
  v *= 100.0f;
  fzoomslider->SetValue((int32)v);
}
void PreviewWindow::SetZoomTextValue(float v)
{
  char str[16];
  v *= 100.0f;
  sprintf(str,"%0.2f",v);
  fzoomtext->SetText(str);
}
void PreviewWindow::SetZoomMenuValue(float v)
{
  v *= 100.0f;
  int i = 0;
  while (i < PreviewWindowSpace::Zoom_List_Total)
  {
    if (PreviewWindowSpace::Zoom_List[i] == v)
    {
      fmn_zoom->ItemAt(i)->SetMarked(true);
      break;      
    }
    i++;
  }
}
