/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#ifndef PREVIEWWINDOW_H
#define PREVIEWWINDOW_H
#include <Window.h>
#include <Message.h>
#include <View.h>
#include <Box.h>
#include <Button.h>
#include <Slider.h>
#include <TextControl.h>
#include <StringView.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <ScrollView.h>

#include <String.h>
#include <OS.h>
#include <Bitmap.h>

#include <stdio.h>

#include "SpoolFile.h"
#include "GUIUtils.h"
#include "PreviewView.h"
#include "PageDefs.h"

namespace PreviewWindowSpace
{
  char  const WindowTitle[] 		= "Print Preview";
  int32 const DefWidth				= 600;
  int32 const DefHeight		 		= 400;
  int32 const TopPanelHeight		= 85;
  int32 const ButtonPanelWidth		= 600;
 
  char const Label_Button_Print[]	= "Print";
  char const Label_Button_Cancel[]	= "Cancel";
  char const Label_Button_Next[]	= ">";
  char const Label_Button_Prev[]	= "<";
  
  char const Label_Scale_Zoom[]		= "Scaling";
  char const Label_Select_Zoom[]	= "<Zoom>";
  char const Label_Page_Format[]	= "Page format:";
  char const Label_Page_Number[]	= "Page:";

  float const Zoom_List[]			= {10.0,25.0,50.0,100.0,200.0,300.0};
  int32 const Zoom_List_Total		= 6;
  int32 const Zoom_List_Default		= 3;
  
  uint32 const MSGCODE_BTN_PRINT	= 'msg0';
  uint32 const MSGCODE_BTN_CANCEL	= 'msg0' + 1;
  uint32 const MSGCODE_BTN_NEXT		= 'msg0' + 2;
  uint32 const MSGCODE_BTN_PREV		= 'msg0' + 3;
  uint32 const MSGCODE_SLIDER_ZOOM	= 'msg0' + 4;
  uint32 const MSGCODE_TEXT_ZOOM	= 'msg0' + 5;
  uint32 const MSGCODE_MENU_ZOOM	= 'msg0' + 6;
  
  int32 const ZOOM_MIN_VALUE		= 5;
  int32 const ZOOM_MAX_VALUE		= 500;
};
class PreviewWindow : public BWindow
{
  public:
    PreviewWindow(SpoolFile *);											// done  
    bool Go(bool inch_mode);											// done
  protected:
    void MessageReceived(BMessage *);									// done
    virtual bool QuitRequested(void);									// done
  private:
    void BuildGUI();													// done
    // msg handling
    void OnPrintBtn();													// done
    void OnCancelBtn();													// done
    void OnNextBtn();													// done
    void OnPrevBtn();													// done
    void OnSliderZoom();												// done
    void OnTextZoom();													// done
    void OnMenuZoom();													// done
    // internal 
    void Uninit();														// done
    void ReadDocumentConfig();											// done
    void ReadPage();													// not 		
    void UpdateNavButtons();											// done
    void UpdatePageNumLabel();											// done
    void UpdatePageFormat(float width,float height,bool inch_meansure);	// done
    void SetZoomSliderValue(float);										// done
    void SetZoomTextValue(float);										// done
    void SetZoomMenuValue(float);
  private:
    sem_id fsem;
    bool  fresult;
    // document info
    int32 fpage_total;
    int32 fpage_current;
    SpoolFile *fspool_file;
    PreviewSpoolBitmap *fpreview_bitmap;
    // GUI value
    BButton *fprintbtn;
    BButton *fcancelbtn;
    BButton *fnextbtn;
    BButton *fprevbtn;
    BSlider *fzoomslider;
    BTextControl *fzoomtext;
    BStringView  *fpageformat;
    BStringView  *fpagenumber;
    BPopUpMenu   *fmn_zoom;    
    PreviewView *fpreviewview;
};
#endif