/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "PrinterSetupWindow.h"
PrinterSetupWindow::PrinterSetupWindow(const char *driver_name)
:BWindow(CenterOfScreen(PrinterSetupWindowSpace::DEF_WIDTH,PrinterSetupWindowSpace::DEF_HEIGHT),driver_name,B_TITLED_WINDOW,B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
,fprinter_info(NULL)
{
  // Build GUI()
  BRect bound = Bounds();
  bound.left--;
  bound.top--;
  bound.bottom++;
  bound.right++;    

  // create a background box
  BBox *mainbox = new BBox(bound);
  AddChild(mainbox);
  
  // create a box for listview
  BRect r = BRect(15,10,bound.Width() - 15,10 + 110);
  BBox *tempbox = new BBox(r,PrinterSetupWindowSpace::LB_PRINTER_MODEL,B_FANCY_BORDER);
  tempbox->SetLabel(PrinterSetupWindowSpace::LB_PRINTER_MODEL);
  mainbox->AddChild(tempbox);
  
  // create a list view
  BRect r1 = BRect(15,20,r.Width() - (15 + B_V_SCROLL_BAR_WIDTH),r.Height() - 15);
  
  lv_model = new BListView(r1,"Model",B_SINGLE_SELECTION_LIST);
  lv_model->SetSelectionMessage(new BMessage(PrinterSetupWindowSpace::MSGCODE_SELECTION_CHANGE));
  lv_model->SetTarget(this);
  
  tempbox->AddChild(new BScrollView("scroll_view",lv_model,B_FOLLOW_LEFT | B_FOLLOW_TOP,0,false,true));
  
  // create a box for unit
  r1 = BRect(r.left,r.bottom + 10,r.left + 70,r.bottom + 85);
  tempbox = new BBox(r1,PrinterSetupWindowSpace::LB_UNIT,B_FANCY_BORDER);
  tempbox->SetLabel(PrinterSetupWindowSpace::LB_UNIT);
  // create radio INCH button
  BRect r3 = BRect(10,20,r1.Width() - 10,35);
  rb_inch = new BRadioButton(r3,NULL,PrinterSetupWindowSpace::LB_INCH,NULL);
  tempbox->AddChild(rb_inch);
  r3.top += 25;
  r3.bottom += 25;
  // create radio CM button
  rb_cm = new BRadioButton(r3,NULL,PrinterSetupWindowSpace::LB_CM,NULL);
  tempbox->AddChild(rb_cm);
  mainbox->AddChild(tempbox);
  
  // create a box for memory used
  BRect r2 = BRect(r1.right + 10,r1.top,r.right,r1.bottom);
  tempbox = new BBox(r2,PrinterSetupWindowSpace::LB_MEMORY,B_FANCY_BORDER);
  tempbox->SetLabel(PrinterSetupWindowSpace::LB_MEMORY);
  // create slider
  r3 = BRect(10,20,r2.Width() - 10,r2.Height() - 20);
  sd_memory = new BSlider(r3,"",PrinterSetupWindowSpace::LB_PHYSICAL_MEM,NULL,0,100,B_TRIANGLE_THUMB);
  sd_memory->SetHashMarks(B_HASH_MARKS_BOTTOM);
  sd_memory->SetHashMarkCount(10);
  const rgb_color color = {0,0,255,0};
  sd_memory->UseFillColor(true,&color);
  sd_memory->SetValue(100);
  sd_memory->SetLimitLabels(PrinterSetupWindowSpace::LB_MIN_PERCENTAGE,PrinterSetupWindowSpace::LB_MAX_PERCENTAGE);
  sd_memory->SetKeyIncrementValue(10);
  sd_memory->SetStyle(B_TRIANGLE_THUMB);
  tempbox->AddChild(sd_memory);
  
  mainbox->AddChild(tempbox);
  // create dont save button
  r1 = BRect(bound.Width() - 180,r2.bottom + 10,bound.Width() - 100,r2.bottom + 30);
  btn_cancel = new BButton(r1,NULL,PrinterSetupWindowSpace::LB_BTN_DONT_SAVE,new BMessage(PrinterSetupWindowSpace::MSGCODE_CANCEL));
  mainbox->AddChild(btn_cancel);
  // create save button
  r1.left = r1.right + 5;
  r1.right = bound.Width() - 14;
  btn_ok = new BButton(r1,NULL,PrinterSetupWindowSpace::LB_BTN_SAVE,new BMessage(PrinterSetupWindowSpace::MSGCODE_OK));
  mainbox->AddChild(btn_ok);  
  // initialize
  LoadModelList();
  if (lv_model->CountItems() > 0)
  {
    select_index = 0;
    lv_model->Select(select_index);
    rb_inch->SetValue(true);
  }
  else
  {
    select_index = -1;
    btn_ok->SetEnabled(false);
  }
}
void PrinterSetupWindow::Quit()
{
  if (fsem >= 0)
  release_sem(fsem);
  BWindow::Quit();  
}
void PrinterSetupWindow::MessageReceived(BMessage *msg)
{
  switch (msg->what)
  {
    case PrinterSetupWindowSpace::MSGCODE_SELECTION_CHANGE:
    OnChange();
    break;
    case PrinterSetupWindowSpace::MSGCODE_OK:
    OnOK();
    break;
    case PrinterSetupWindowSpace::MSGCODE_CANCEL:
    OnCancel();
    break;
    default:
    BWindow::MessageReceived(msg);  
  }
}
void PrinterSetupWindow::OnChange()
{
  Lock();
  int32 cur_sel = lv_model->CurrentSelection();
  if (cur_sel >= 0)
  select_index = cur_sel;
  else
  lv_model->Select(select_index);
  Unlock();
}
void PrinterSetupWindow::OnOK()
{
  BPath path;
  Lock();
  // save result
  *fresult = B_OK; 
  // save printer model
  BStringItem *item = (BStringItem *) lv_model->ItemAt(select_index);
  if (find_directory(B_COMMON_ADDONS_DIRECTORY,&path,true) == B_OK)
  {
    path.Append("Be Universal Printer Addon");
    path.Append(item->Text());
    fprinter_info->PrinterModel(path.Path());
  }
  
  // save printer unit  
  if (rb_inch->Value())
  fprinter_info->Unit(PRINTER_UNIT_INCH);
  else
  fprinter_info->Unit(PRINTER_UNIT_CM);
  // save printer memory
  // nothing
 
  release_sem(fsem);
  Unlock();
}
void PrinterSetupWindow::OnCancel()
{
  Lock();
  release_sem(fsem);
  Unlock();
}
status_t PrinterSetupWindow::Go(PrinterInfo *printer_info)
{
  sem_id sem;
  status_t result = B_OK + 1;
  // create sem
  sem = create_sem(1,"printer_setup_sem");
  if (sem < 0)
  goto err_sem;
  this->fsem = sem;
  this->fresult = &result;
  this->fprinter_info = printer_info;
  // acquire_sem 
  acquire_sem(sem);
  // Show Window
  this->Show();
  // acquire_sem
  acquire_sem(sem);
  delete_sem(sem);
  this->Lock();
  this->Quit();  
  return result;

  err_sem:
  return sem; 
}    
void PrinterSetupWindow::LoadModelList()
{
  char buf[B_FILE_NAME_LENGTH];
  BPath path;
  BEntry entry;
  BPath addonpath;
  struct stat st;
  // go to directory 
  if (find_directory(B_COMMON_ADDONS_DIRECTORY,&path,true) != B_OK)
  return;
  path.Append("Be Universal Printer Addon");
  BDirectory *dir = new BDirectory(path.Path());
  // reading every node in this directory
  dir->Rewind();
  while (dir->GetNextEntry(&entry) == B_OK)
  {
    if (entry.GetStat(&st) == B_OK)
    {
      memset(buf,0,sizeof(buf));
      if (S_ISREG(st.st_mode) && (entry.GetName(buf) == B_OK))
      {
        int len = strlen(buf);
        if (len < (int)sizeof(buf))
        buf[len] = '\0';
        lv_model->AddItem(new BStringItem(buf));
      }
    } 
  };
}