/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include "driver.h"
#include <stdio.h>
#include <memory.h>
#include <StorageKit.h>
#include <Directory.h>

#include <PrinterInfo.h>
#include <PrinterAddon.h>
#include <PageDefs.h>
#include "DebugUtils.h"
#include "PrinterSetupWindow.h"
#include "PageSetupWindow.h"
#include "JobSetupWindow.h"
#include "SpoolFile.h"
#include "PreviewWindow.h"

// -------------------------------------------------------------------
// TAKE JOB
BMessage *take_job(BFile *spool_file,BNode *spool_dir,BMessage *msg)
{
  SpoolFile *sf;
  PrinterInfo  printer_info;
  PreviewWindow *preview_win;
  PrinterAddon printer_addon;
  bool print = true;

  // Read Printer Info
  printer_info.Read(spool_dir);
  
  sf = new SpoolFile();
  
  // Read in the spool file
  if (!sf->SetTo(spool_file))
  goto error;

  if (sf->Preview())
  {
    preview_win = new PreviewWindow(sf);
    print = preview_win->Go(printer_info.Unit() == PRINTER_UNIT_INCH);
  }

  if (print)
  {
    // load add on printer module
    if (printer_addon.LoadAddon(printer_info.PrinterModel()) == B_OK)
    printer_addon.PrintDocument(&printer_info,sf);    
  }
  
  error:
  delete sf;
  msg->what = 'okok';
  // Remove "_spool/Status" attr
  // so the print_server will delete this spool file
  spool_file->RemoveAttr("_spool/Status");
  spool_file->Sync();
  spool_file->Unlock();
  return NULL;
}
// -------------------------------------------------------------------

// -------------------------------------------------------------------
// CONFIG Page
BMessage *config_page(BNode *spool_dir,BMessage *msg)
{
  
  PageSetupWindow *page_setup_win;
  BMessage *pagesetup_msg;
  PrinterAddon addon;
  PrinterInfo  printer_info;

  printer_info.Read(spool_dir);
  if (addon.LoadAddon(printer_info.PrinterModel()) != B_OK)
  return NULL;
  
  pagesetup_msg = new BMessage(*msg);
  page_setup_win = new PageSetupWindow(&addon,printer_info.Unit());
  page_setup_win->Go(pagesetup_msg);
  if (pagesetup_msg->what == 'okok')
  return pagesetup_msg;  

  delete pagesetup_msg;
  return NULL;   
}
// -------------------------------------------------------------------

// -------------------------------------------------------------------
// Config Job
BMessage *config_job(BNode *spool_dir,BMessage *msg)
{
  JobSetupWindow *job_setup_win;
  BMessage *jobsetup_msg;
  PrinterAddon *printer_addon;
  PrinterInfo  *printer_info;
  printer_info = new PrinterInfo;
  printer_info->Read(spool_dir);
  printer_addon = new PrinterAddon;
  
  if (printer_addon->LoadAddon(printer_info->PrinterModel()) != B_OK)
  {
    delete printer_info;
    delete printer_addon;
    return NULL;
  }

  jobsetup_msg = new BMessage(*msg);

  job_setup_win = new JobSetupWindow(printer_info->PrinterName(),printer_addon,printer_info,jobsetup_msg);
  job_setup_win->Go();
  delete printer_info;
  delete printer_addon;
  if (jobsetup_msg->what != 'okok')
  {
    delete jobsetup_msg;
    return NULL;
  }
  return jobsetup_msg;
}
// -------------------------------------------------------------------


// -------------------------------------------------------------------
// Add Printer
char *add_printer(char *printer_name)
{
  BDirectory *node;
  PrinterSetupWindow *printer_setup;
  BPath path;
  PrinterInfo printer_info;
   
  // show printer model dialog
  printer_setup = new PrinterSetupWindow(printer_name);
  
  if (printer_setup == NULL)
  goto err_0;
  
  if (printer_setup->Go(&printer_info) == B_OK)
  {
    // find the directory
    if (find_directory(B_COMMON_SETTINGS_DIRECTORY,&path,true) != B_OK)
    goto err_0;
    // append path 
    path.Append("printers");
    path.Append(printer_name);
    // now open the node
    node = new BDirectory(path.Path());
    // save printer info into printer node attributes
    node->Lock();
    printer_info.Write(node);
    node->Unlock();
    delete node;
    return printer_name;
  }
  // err 0
  err_0:
  return NULL;  
}
// -------------------------------------------------------------------