/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include <PrinterAddon.h>
PrinterAddon::PrinterAddon()
:fimageid(-1)
,fminpagewidth(0)
,fminpageheight(0)
,fmaxpagewidth(0)
,fmaxpageheight(0)
,fauthor("")
,fsupportpapermedia(NULL)
,fqualitycolor(NULL)
,fqualitymono(NULL)
{

}
PrinterAddon::~PrinterAddon()
{
  UnloadAddon();
}
void PrinterAddon::UnloadAddon()
{
  if (fimageid >= B_OK)
  {
    unload_add_on(fimageid);
    UninitValue();
  }
}
void PrinterAddon::UninitValue()
{    
  fimageid = -1;
    
  fauthor = "";
    
  fqualitycolor = NULL;
    
  fqualityindexcolor = NULL;

  fqualitymono  = NULL;
    
  fqualityindexmono = NULL;
    
  fpaper_feeder = NULL;
    
  fsupportpapermedia = NULL;

  fminpagewidth  = 0;
  fminpageheight = 0;
  fmaxpagewidth  = 0;
  fmaxpageheight = 0;
  fnumber_of_ink = 0;

  fsupportcolorprint = false;

  fnozzlecheck = NULL;

  fheadclean   = NULL;
  fchangecartridge    = NULL;
  finkinfo     = NULL;
  finklevel    = NULL;
  fprintdoc    = NULL;
}
status_t PrinterAddon::LoadAddon(const char *path)
{
  UnloadAddon();
  fimageid = load_add_on(path);
  if (fimageid < B_OK)
  {
    UnloadAddon();
    return B_ERROR;
  }
  // now testing the addon
  if (!VerifyAddon())
  {
    UnloadAddon();
    return B_ERROR;
  }
  return B_OK;
}
status_t PrinterAddon::InitCheck()
{
  if (fimageid < B_OK)
  return B_ERROR;
  return B_OK;
}
uint32 PrinterAddon::MinPageWidth()
{
  return fminpagewidth;
}
uint32 PrinterAddon::MinPageHeight()
{
  return fminpageheight;
}
uint32 PrinterAddon::MaxPageWidth()
{
  return fmaxpagewidth;
}
uint32 PrinterAddon::MaxPageHeight()
{
  return fmaxpageheight;
}

paper_feeder_st *PrinterAddon::PaperFeeder()
{
  return fpaper_feeder;
}

uint8 PrinterAddon::NumberOfInks()
{
  return fnumber_of_ink;
}

const char *PrinterAddon::Author()
{
  return fauthor.String();
}

bool PrinterAddon::SupportColorPrint()
{
  return fsupportcolorprint;
}
print_quality_st *PrinterAddon::SupportQualityColor()
{
  return fqualitycolor;
}
print_quality_index_st *PrinterAddon::SupportQualityIndexColor()
{
  return fqualityindexcolor;
}

print_quality_st *PrinterAddon::SupportQualityMono()
{
  return fqualitymono;
} 
print_quality_index_st *PrinterAddon::SupportQualityIndexMono()
{
  return fqualityindexcolor;
}

support_paper_media_st *PrinterAddon::SupportPaperMedia()
{
  return fsupportpapermedia;
}
bool PrinterAddon::VerifyAddon()
{
  void *func;
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_MINIMUNPAGEWIDTH,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fminpagewidth = *((uint32 *) func);   
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_MINIMUNPAGEHEIGHT,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fminpageheight = *((uint32 *) func);   

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_MAXIMUNPAGEWIDTH,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fmaxpagewidth = *((uint32 *) func);   

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_MAXIMUNPAGEHEIGHT,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fmaxpageheight = *((uint32 *) func);   

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_NUMOFINKS,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fnumber_of_ink = *((uint8 *) func);   
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_PAPER_FEEDER,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fpaper_feeder = ((paper_feeder_st *) func);   
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_AUTHOR,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fauthor = ((const char *) func);

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_COLOR_PRINT,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  return false;
  fsupportcolorprint = *((bool *) func);
 
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_PAPER_MEDIA,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  fsupportpapermedia = NULL;
  else
  fsupportpapermedia = (support_paper_media_st *) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_QUALITY_COLOR,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  fqualitycolor = NULL;
  else
  fqualitycolor = (print_quality_st *) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_QUALITY_INDEX_COLOR,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  fqualityindexcolor = NULL;
  else
  fqualityindexcolor = (print_quality_index_st *) func;
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_QUALITY_MONO,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  fqualitymono = NULL;
  else
  fqualitymono = (print_quality_st *) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::DATANAME_SUPPORT_QUALITY_INDEX_MONO,B_SYMBOL_TYPE_DATA,&func) < B_OK)
  fqualityindexmono = NULL;
  else
  fqualityindexmono = (print_quality_index_st *) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_UTILITY_NOZZLECHECK,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  fnozzlecheck = NULL;
  else
  fnozzlecheck = (nozzle_check_func) func;
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_UTILITY_HEADCLEAN,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  fheadclean = NULL;
  else
  fheadclean = (head_clean_func) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_UTILITY_CHANGECARTRIDGE,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  fchangecartridge = NULL;
  else
  fchangecartridge = (change_cartridge_func) func;
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_UTILITY_INKINFO,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  finkinfo = NULL;
  else
  finkinfo = (ink_info_func) func;

  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_UTILITY_INKLEVEL,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  finklevel = NULL;
  else
  finklevel = (ink_level_func) func;
  
  if (get_image_symbol(fimageid,PrinterAddonSpace::FUNCNAME_PRINT_DOCUMENT,B_SYMBOL_TYPE_TEXT,&func) < B_OK)
  fprintdoc = NULL;
  else
  fprintdoc = (print_doc_func) func;
  
  return true;
}

bool PrinterAddon::SupportNozzleCheck()const
{
  return (fnozzlecheck != NULL);
}
bool PrinterAddon::SupportHeadClean()const
{
  return (fheadclean != NULL);
}
bool PrinterAddon::SupportChangeCartridge()const
{
  return (fchangecartridge != NULL);
}
bool PrinterAddon::SupportInkLevel()const
{
  return (finkinfo != NULL);
}
    
printer_utility_return_t PrinterAddon::NozzleCheck(PrinterInfo *pprinter_info)
{
  return fnozzlecheck(pprinter_info);
}
printer_utility_return_t PrinterAddon::HeadClean(PrinterInfo *pprinter_info)
{
  return fheadclean(pprinter_info);
}
printer_utility_return_t PrinterAddon::ChangeCartridge(PrinterInfo *pprinter_info)
{
  return fchangecartridge(pprinter_info);
}
bool PrinterAddon::InkInfo(PrinterInfo *pprinter_info,InkLevelGroup *pinkgroup)
{
  if (finkinfo != NULL)
  return finkinfo(pprinter_info,pinkgroup);
  return true;
}
printer_utility_return_t PrinterAddon::InkLevel(PrinterInfo *pprinter_info,InkLevelGroup *pinkgroup)
{
  return finklevel(pprinter_info,pinkgroup);
}
printer_utility_return_t PrinterAddon::PrintDocument(PrinterInfo *pprinter_info,SpoolFile *pspool_file)
{
  if (fprintdoc != NULL)
  return fprintdoc(pprinter_info,pspool_file);
  return PRINTER_FUNC_NOT_SUPPORT;
}

