/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#ifndef PRINTERADDON_H
#define PRINTERADDON_H
#include <OS.h>
#include <image.h>
#include <String.h>
#include <AddonDefs.h>
#include <PrinterInfo.h>
#include <InkLevelGroup.h>
#include <SpoolFile.h>
namespace PrinterAddonSpace
{
  char const DATANAME_MINIMUNPAGEWIDTH[]			= "MinimunPageWidth";
  char const DATANAME_MINIMUNPAGEHEIGHT[]			= "MinimunPageHeight";
  char const DATANAME_MAXIMUNPAGEWIDTH[]			= "MaximunPageWidth";
  char const DATANAME_MAXIMUNPAGEHEIGHT[]			= "MaximunPageHeight";
  char const DATANAME_NUMOFINKS[]					= "number_of_inks";
  char const DATANAME_SUPPORT_PAPER_FEEDER[]		= "support_paper_feeder";
  char const DATANAME_AUTHOR[]						= "Author";
  char const DATANAME_SUPPORT_COLOR_PRINT[]			= "SupportColorPrint";
  char const DATANAME_SUPPORT_PAPER_MEDIA[]			= "support_paper_media";
  char const DATANAME_SUPPORT_QUALITY_COLOR[]		= "support_quality_color";
  char const DATANAME_SUPPORT_QUALITY_INDEX_COLOR[]	= "support_quality_index_color";
  char const DATANAME_SUPPORT_QUALITY_MONO[]		= "support_quality_mono";
  char const DATANAME_SUPPORT_QUALITY_INDEX_MONO[]	= "support_quality_index_mono";
  
  /* optional */
  char const FUNCNAME_UTILITY_NOZZLECHECK[]			= "nozzlecheck";
  char const FUNCNAME_UTILITY_HEADCLEAN[]			= "headclean";
  char const FUNCNAME_UTILITY_CHANGECARTRIDGE[]		= "changecartridge";
  char const FUNCNAME_UTILITY_INKINFO[]				= "inkinfo";
  char const FUNCNAME_UTILITY_INKLEVEL[]			= "inklevel";
  
  char const FUNCNAME_PRINT_DOCUMENT[]				= "printdoc";
};
typedef enum {PRINTER_OK = 0,PRINTER_ERROR_COMMUNICATION = 1,PRINTER_ERROR_BUSY = 2,PRINTER_USER_ABORT = 3,PRINTER_FUNC_NOT_SUPPORT = 4}printer_utility_return_t;
typedef printer_utility_return_t (*nozzle_check_func) (PrinterInfo*);
typedef printer_utility_return_t (*head_clean_func) (PrinterInfo*);
typedef printer_utility_return_t (*change_cartridge_func) (PrinterInfo*);
typedef bool (*ink_info_func) (PrinterInfo*,InkLevelGroup*);
typedef printer_utility_return_t (*ink_level_func)(PrinterInfo*,InkLevelGroup*);
typedef printer_utility_return_t (*print_doc_func)(PrinterInfo*,SpoolFile*);
class PrinterAddon
{
  public:
    PrinterAddon();
    ~PrinterAddon();
    status_t LoadAddon(const char *path);
    status_t InitCheck();
    uint32 MinPageWidth();
    uint32 MinPageHeight();
    uint32 MaxPageWidth();
    uint32 MaxPageHeight();
    
    paper_feeder_st *PaperFeeder();
    
    const char *Author();
    
    uint8 NumberOfInks();

    bool SupportColorPrint();
    support_paper_media_st *SupportPaperMedia();
    
    print_quality_st *SupportQualityColor();
    print_quality_index_st *SupportQualityIndexColor();
    
    print_quality_st *SupportQualityMono();
    print_quality_index_st *SupportQualityIndexMono();

    bool SupportNozzleCheck()const;
    bool SupportHeadClean()const;
    bool SupportChangeCartridge()const;
    bool SupportInkLevel()const;
   
    printer_utility_return_t NozzleCheck(PrinterInfo*);
    printer_utility_return_t HeadClean(PrinterInfo*);
    printer_utility_return_t ChangeCartridge(PrinterInfo*);
    bool InkInfo(PrinterInfo*,InkLevelGroup*);
    printer_utility_return_t InkLevel(PrinterInfo*,InkLevelGroup*);
    printer_utility_return_t PrintDocument(PrinterInfo*,SpoolFile*);
  private:
    bool VerifyAddon();  
    void UnloadAddon();
    void UninitValue();
  private:
    image_id fimageid;
    uint32 fminpagewidth;
    uint32 fminpageheight;
    uint32 fmaxpagewidth;
    uint32 fmaxpageheight;
    uint8  fnumber_of_ink;
    paper_feeder_st *fpaper_feeder;
    BString fauthor;
    bool    fsupportcolorprint;
    support_paper_media_st *fsupportpapermedia;
    print_quality_st *fqualitycolor;
    print_quality_index_st *fqualityindexcolor;
    print_quality_st *fqualitymono;
    print_quality_index_st *fqualityindexmono;
    nozzle_check_func fnozzlecheck;
    head_clean_func   fheadclean;
    change_cartridge_func    fchangecartridge;
    ink_info_func	  finkinfo;
    ink_level_func    finklevel;
    print_doc_func    fprintdoc;
};
#endif