/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#include <RasterSpool.h>
/* =================================================================== */
RasterSpool::RasterSpool()
:ScaleInfo()
{
  UnsetValue();
}
RasterSpool::~RasterSpool()
{
  Uninit();
}
/* =================================================================== */

/* =================================================================== */
BRect RasterSpool::PrintableFrame()
{
  return fprintableframe;
}
BRect RasterSpool::ScaningFrame()
{
  return fscanframe;
}
BRect RasterSpool::ScaningRect()
{
  return GetRectBounds(fscanframe);
}
/* =================================================================== */

/* =================================================================== */
int RasterSpool::PageNumber()
{
  return fpagenumber;
}
/* =================================================================== */

/* =================================================================== */
void RasterSpool::UnsetValue()
{
  fpagenumber = 0;
  
  fprintableframe  = RasterSpoolSpace::EMPTYRECT;
  fprintablebounds = RasterSpoolSpace::EMPTYRECT;
  
  fscanframe     = RasterSpoolSpace::EMPTYRECT;
  fpostscanframe = RasterSpoolSpace::EMPTYRECT;
  fbufferrect	 = RasterSpoolSpace::EMPTYRECT;
  
  fspoolfile = NULL;
  fbitmap    = NULL;
  finit = false;
}
BRect RasterSpool::GetRectBounds(BRect rect)
{
  return BRect(0,0,rect.Width(),rect.Height());
}
BRect RasterSpool::RotateRect(BRect rect)
{
  return BRect(rect.top,rect.left,rect.bottom,rect.right);
}
void RasterSpool::PreCalculatePostScanFrame()
{
  int reminder = (int32) (fprintablebounds.bottom - fscanframe.bottom);
  if (reminder <= 0)
  fendofscan = true;
  else if (reminder < fbufferrect.IntegerHeight())
  {
    fpostscanframe = fbufferrect;
    fpostscanframe.top = fscanframe.bottom;
    fpostscanframe.bottom = fprintablebounds.bottom;
  }
  else
  {
    fpostscanframe = fbufferrect;
    fpostscanframe.OffsetTo(0,fscanframe.bottom);
  }
}	
void RasterSpool::UninitBitmap()						
{
  if (fbitmap != NULL)
  {
    delete fbitmap; 
    fbitmap = NULL;
  }
}	
/* =================================================================== */

/* =================================================================== */
void RasterSpool::Uninit()
{
  if (InitOK())
  {
    UninitBitmap();
    UnsetValue();
    ScaleInfo::Unset();
  }
}
bool RasterSpool::InitOK()
{
  return finit;
}
bool RasterSpool::Init(SpoolFile *spoolfile,int maxline)
{
  // uninit the object if it's already been initialized
  Uninit();
  
  // check the maxline parameter if is less than one then return failure
  if (maxline < 1)
  return InitOK();
  
  // assign internal fspoolfile object to spoolfile parameter  
  fspoolfile = spoolfile;
  // inital scale info with xres and yres from fspoolfile parameter
  ScaleInfo::SetTo(fspoolfile->XResolution(),fspoolfile->YResolution());  

  // calculate printable_frame at draw_scale 
  fprintableframe = ScaleInfo::NonScaleToDrawScale(fspoolfile->PrintableRect());
  // rotate the printable_frame if spoolfile option in landscape
  if (fspoolfile->Landscape())
  fprintableframe = RotateRect(fprintableframe);
  // calculate fprintablebounds from fprintableframe;
  fprintablebounds = GetRectBounds(fprintableframe);
  
  // calculate buffer rect
  if (maxline > fprintablebounds.IntegerWidth())
  fbufferrect = BRect(0,0,fprintablebounds.IntegerWidth(),fprintablebounds.IntegerHeight());
  else
  fbufferrect = BRect(0,0,fprintablebounds.IntegerWidth(),maxline);
  
  fpagenumber = 0;
  finit = true;
    
  return InitOK();
}
/* ========================================================= */

/* ========================================================= */
bool RasterSpool::SetToPage(int pagenumber)
{
  if (!InitOK())
  return false;

  // check page range pagenumber > 0 and pagenumber <= page_count;
  // page number start at 1 not 0  
  if ((pagenumber < 1) || (pagenumber > fspoolfile->PrintFileHeader().page_count))
  {
    fendofscan = true;
    return false;
  }

  fendofscan = false;
  fpagenumber = pagenumber;
  
  fscanframe     = RasterSpoolSpace::EMPTYRECT;
  fpostscanframe = fbufferrect;

  // calculate page rect (screen scale mode)  
  fscreenscale_pagerect = GetRectBounds(fspoolfile->PrintableRect());
  fscreenscale_pagerect.OffsetTo(0,fscreenscale_pagerect.Height() * (PageNumber() - 1));
  
  fprintscale_pagerect = ScaleInfo::NonScaleToDrawScale(fscreenscale_pagerect);
  // uninitialize bitmap 
  UninitBitmap();

  return true;
}

bool RasterSpool::ScanNext()
{
  BRect  drawscale_printable_rect;
  BRect  drawing_frame;

  BRect  bitmap_rect;
  BRect  drawing_constrain_rect;
  BPoint drawing_origin_point;
  
  BView *view;
  
  // --------------------------------------
  UninitBitmap();
    
  // check end of scan if yes return false;
  if (fendofscan)
  return false;

  // set scan frame to post calculated scan frame  
  fscanframe = fpostscanframe;

  // calculate DrawScale Printable Rect
  drawscale_printable_rect = fprintablebounds;

  // calculate drawing frame
  if (fspoolfile->Portrait())
  {
    if (!fspoolfile->Rotate180())
    drawing_frame = fscanframe;
    else
    drawing_frame = BRect(
                           fscanframe.left,
                           (drawscale_printable_rect.bottom - fscanframe.top) - fscanframe.Height(),
                           fscanframe.right,
                           drawscale_printable_rect.bottom - fscanframe.top
                         );
  }
  else
  {
    if (!fspoolfile->Rotate180())
    drawing_frame = RotateRect(fscanframe);
    else
    drawing_frame = BRect(
                           (drawscale_printable_rect.bottom - fscanframe.top) - fscanframe.Height(),
                           fscanframe.left,
                           drawscale_printable_rect.bottom - fscanframe.top,
                           fscanframe.right
                         );
  }

  bitmap_rect = GetRectBounds(drawing_frame);
  
  // calculate drawing_constrain_rect
  drawing_constrain_rect = drawing_frame;
  drawing_constrain_rect.left = ScaleInfo::DrawScaleXToNonScale(drawing_constrain_rect.left);
  drawing_constrain_rect.top  = ScaleInfo::DrawScaleYToNonScale(drawing_constrain_rect.top);
  drawing_constrain_rect.OffsetTo(fscreenscale_pagerect.left,fscreenscale_pagerect.top);

  // calculate drawing_origin_point
  drawing_origin_point.x = -(DrawScaleXToNonScale(drawing_frame.left) + fscreenscale_pagerect.left);
  drawing_origin_point.y = -(DrawScaleYToNonScale(drawing_frame.top) + fscreenscale_pagerect.top);
  // ===========================================
  // drawing things
  fbitmap = new BBitmap(bitmap_rect,B_RGBA32,true,false);
  view = new BView(bitmap_rect,NULL,B_FOLLOW_NONE,B_WILL_DRAW);
  fbitmap->Lock();
  fbitmap->AddChild(view);
  // fill background
  view->SetHighColor(255,255,255);
  view->FillRect(view->Bounds());

  view->SetScale(ScaleInfo::DrawScale());

  // set view origin point
  view->SetOrigin(drawing_origin_point);

  // set view draw constrain clipping region  
  BRegion *draw_region = new BRegion(drawing_constrain_rect);
  view->ConstrainClippingRegion(draw_region);
  delete draw_region;

  for (int i = 0;i < fspoolfile->CountPictures();i++)
  {
    PictureSpool *pic = fspoolfile->GetPicture(i);
    if (fscreenscale_pagerect.Contains(pic->Frame()))
    view->DrawPicture(pic->Picture(),pic->Frame().LeftTop());
  }

  fbitmap->RemoveChild(view);
  fbitmap->Unlock();
  delete view;
  // ===========================================
  
  // apply special effect 
  if (fspoolfile->Portrait())
  {
    if (fspoolfile->Rotate180())
    {
      if (fspoolfile->ColorPrint())
      SpecialEffectPortraitRotate180();
      else
      SpecialEffectPortraitRotate180ToGrayScale();
    }
    else
    {
      if (!fspoolfile->ColorPrint())
      SpecialEffectPortraitToGrayScale();
    }
  }
  else
  {
    if (fspoolfile->Rotate180())
    {
      if (fspoolfile->ColorPrint())
      SpecialEffectLandscapeRotate180();
      else
      SpecialEffectLandscapeRotate180ToGrayScale();
    }
    else
    {
      if (fspoolfile->ColorPrint())
      SpecialEffectLandscape();
      else
      SpecialEffectLandscapeToGrayScale();
    }
  }    
  // calculate post scan frame;  
  PreCalculatePostScanFrame();
  return true;
}
/* =================================================================== */

/* =================================================================== */
void RasterSpool::SpecialEffectPortraitRotate180()
{
  rgb_color color_value;
  BRect src_rect = fbitmap->Bounds();
  BBitmap *new_bitmap = new BBitmap(src_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < src_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < src_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,src_rect.IntegerWidth() - i,src_rect.IntegerHeight() - j,&color_value);
      WriteColorPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;
}
void RasterSpool::SpecialEffectPortraitRotate180ToGrayScale()
{
  rgb_color color_value;
  BRect src_rect = fbitmap->Bounds();
  BBitmap *new_bitmap = new BBitmap(src_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < src_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < src_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,src_rect.IntegerWidth() - i,src_rect.IntegerHeight() - j,&color_value);
      WriteGrayPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;
}
void RasterSpool::SpecialEffectPortraitToGrayScale()
{
  rgb_color color_value;
  BRect src_rect = fbitmap->Bounds();
  for (int j = 0;j < src_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < src_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,i,j,&color_value);
      if (!IsWhite(color_value))
      WriteGrayPixel(fbitmap,i,j,color_value);    
    }
  }
}
/* =================================================================== */

/* =================================================================== */
void RasterSpool::SpecialEffectLandscapeRotate180()
{
  rgb_color color_value;
  BRect des_rect = RotateRect(fbitmap->Bounds());
  int bitmap_width = des_rect.IntegerWidth();
  BBitmap *new_bitmap = new BBitmap(des_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < des_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < des_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,j,bitmap_width - i,&color_value);
      WriteColorPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;  
}
void RasterSpool::SpecialEffectLandscapeRotate180ToGrayScale()
{
  rgb_color color_value;
  BRect des_rect = RotateRect(fbitmap->Bounds());
  int bitmap_width = des_rect.IntegerWidth();
  BBitmap *new_bitmap = new BBitmap(des_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < des_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < des_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,j,bitmap_width - i,&color_value);
      WriteGrayPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;
}
void RasterSpool::SpecialEffectLandscape()
{
  rgb_color color_value;
  BRect des_rect = RotateRect(fbitmap->Bounds());
  int bitmap_height = des_rect.IntegerHeight();
  BBitmap *new_bitmap = new BBitmap(des_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < des_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < des_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,bitmap_height - j,i,&color_value);
      WriteColorPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;
}
void RasterSpool::SpecialEffectLandscapeToGrayScale()
{
  rgb_color color_value;
  BRect des_rect = RotateRect(fbitmap->Bounds());
  int bitmap_height = des_rect.IntegerHeight();
  BBitmap *new_bitmap = new BBitmap(des_rect,fbitmap->ColorSpace(),false,false);
  for (int j = 0;j < des_rect.IntegerHeight();j++)
  {
    for (int i = 0;i < des_rect.IntegerWidth();i++)
    {
      RetrievePixel(fbitmap,bitmap_height - j,i,&color_value);
      WriteGrayPixel(new_bitmap,i,j,color_value);    
    }
  }
  delete fbitmap;
  fbitmap = new_bitmap;
}
/* =================================================================== */

/* =================================================================== */
inline void RasterSpool::RetrievePixel(BBitmap *bitmap,int x,int y,rgb_color *color_value)
{
  *color_value = *((rgb_color *) ((char *) bitmap->Bits() + ((y * bitmap->BytesPerRow()) + (x * sizeof(rgb_color)))));
}
inline void RasterSpool::WriteGrayPixel(BBitmap *bitmap,int x,int y,rgb_color color_value)
{
  color_value.red = color_value.green = color_value.blue = 
  (color_value.red + color_value.green + color_value.blue) / 3;
  WriteColorPixel(bitmap,x,y,color_value);
}
inline void RasterSpool::WriteColorPixel(BBitmap *bitmap,int x,int y,rgb_color color_value)
{
  *((rgb_color *) ((char *) bitmap->Bits() + ((y * bitmap->BytesPerRow()) + (x * sizeof(rgb_color))))) = color_value;
}
inline bool RasterSpool::IsWhite(rgb_color color_value)
{
  ((color_value.red == 255) && (color_value.green == 255) && (color_value.blue == 255));
}
/* =================================================================== */

BBitmap *RasterSpool::Bitmap()
{
  return fbitmap;
}
