#include "SpoolFile.h"
#include <Application.h>
SpoolFile::SpoolFile()
:SpoolFileHeader(),ffile(NULL),fpiclist(NULL)
{
  fpiclist = new BList();
}
SpoolFile::~SpoolFile()
{
  Unset();
  delete fpiclist;
}
bool SpoolFile::SetTo(BFile *spool_file)
{
  BRect rect;
  BPicture *pic;

  if (!SpoolFileHeader::SetTo(spool_file))
  goto error;

  while (spool_file->Seek(60,SEEK_CUR) >= 0)
  {
    // read BRect
    if (spool_file->Read(&rect,sizeof(rect)) < (ssize_t) sizeof(rect))
    break;
    
    // allocated new BPicture
    pic = new BPicture();
    if (pic == NULL)
    break;
    
    if (pic->Unflatten(spool_file) != B_OK)
    {
      delete pic;
      break;
    }
    fpiclist->AddItem(new PictureSpool(pic,rect));    
  }
  return true;
  
  error:
  Unset();
  return false;
}
void SpoolFile::Unset()
{
  if (InitOK())
  {
    for (int i = 0;i < fpiclist->CountItems();i++)
    delete (PictureSpool *) fpiclist->ItemAt(i);

    fpiclist->MakeEmpty();
    
    if (ffile != NULL)
    {
      ffile->Unset();
      delete ffile;
      ffile = NULL;
    }

    SpoolFileHeader::Unset();
  }
}
void SpoolFile::ReadPreviewPage(int page_index,PreviewSpoolBitmap *preview_bitmap)
{
  BBitmap *bitmap;
  BView *view;
  BRect page_rect;
  BRect paper_rect;
  BRect printable_rect;
  BPoint printable_offset;  
  if (ReverseOrder())
  page_index = ReversePageNumber(page_index);
  paper_rect = BRect(0,0,PaperRect().Width(),PaperRect().Height());
  printable_rect = BRect(0,0,PrintableRect().Width(),PrintableRect().Height());
  printable_offset = BPoint( 
                             (paper_rect.Width() - printable_rect.Width()) / 2,
                             (paper_rect.Height() - printable_rect.Height()) / 2);

  // Initialize Destination Bitmap Object
  if (Landscape() || Rotate180())
  bitmap = new BBitmap (paper_rect,B_RGBA32,true,false);
  else
  bitmap = preview_bitmap->Bitmap();

  /* =================================================================== */  
  printable_offset = BPoint( 
                             (paper_rect.Width() - printable_rect.Width()) / 2,
                             (paper_rect.Height() - printable_rect.Height()) / 2);

  page_rect = printable_rect;
  page_rect.top    = (floor(printable_rect.Height()) * (page_index - 1));
  page_rect.bottom = (floor(printable_rect.Height()) * page_index);

  // create view to handling drawing
  view = new BView(paper_rect,NULL,B_FOLLOW_NONE,B_WILL_DRAW);
  
  bitmap->Lock();    
  // bitmap add view as its child
  bitmap->AddChild(view);

  BRect draw_rect = printable_rect;
  draw_rect.OffsetTo(printable_offset);

  // fill background 
  view->SetHighColor(255,255,255);
  view->FillRect(view->Bounds());
  BPoint k = page_rect.LeftTop() - printable_offset;
  view->SetOrigin(-floor(k.x),-floor(k.y));

  BRegion *draw_region = new BRegion(page_rect);
  view->ConstrainClippingRegion(draw_region);
  delete draw_region;
  
  for (int i = 0;i < CountPictures();i++)
  {
    PictureSpool *pic = GetPicture(i);
    if (page_rect.Contains(pic->Frame()))
    view->DrawPicture(pic->Picture(),pic->Frame().LeftTop());
  }
  bitmap->RemoveChild(view);
  bitmap->Unlock();
  delete view;
  /* =================================================================== */  
  
  /* =================================================================== */  
  if (Landscape())
  {
    if (Rotate180())
    {
      if (ColorPrint())
      FlipImage270(preview_bitmap->Bitmap(),bitmap);
      else
      FlipImage270ToBW(preview_bitmap->Bitmap(),bitmap);
    }
    else
    {
      if (ColorPrint())
      FlipImage90(preview_bitmap->Bitmap(),bitmap);
      else
      FlipImage90ToBW(preview_bitmap->Bitmap(),bitmap);
    }
    delete bitmap;
  }
  else
  {
    if (Rotate180())
    {
      if (ColorPrint())
      FlipImage180(preview_bitmap->Bitmap(),bitmap);
      else
      FlipImage180ToBW(preview_bitmap->Bitmap(),bitmap);
      delete bitmap;
    }
    else
    {
      if (!ColorPrint())
      ConvertToBW(preview_bitmap->Bitmap());
    }
  }    
  /* =================================================================== */  
}
void SpoolFile::ToBW(rgb_color& color)
{
  color.red = color.green = color.blue = 
  (color.red + color.green + color.blue) / 3;
}
void SpoolFile::ConvertToBW(BBitmap *bitmap)
{
  uint32 *data = (uint32 *) bitmap->Bits();
  int pixels = (int) bitmap->BitsLength() / 4;
  rgb_color color;
  for (int i = 0;i < pixels;i++)
  {
    memcpy(&color,data + i,sizeof(color));
    ToBW(color);
    memcpy(data + i,&color,sizeof(color));
  }
}
void SpoolFile::FlipImage180(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  int32 si,sj,di,dj;
  int32 w = (int32) des_bitmap->Bounds().Width();
  int32 h = (int32) des_bitmap->Bounds().Height();

  sj = 0;
  dj = h - 1;
  
  while ((sj < h) && (dj >= 0))
  {
    si = 0;
    di = w - 1;
    
    while ((si < w) && (di >= 0))
    {
      *((uint32 *) ( ((char *) des_bitmap->Bits()) +  ((dj * des_bitmap->BytesPerRow()) + (di * sizeof(uint32))))) 
      =
      *((uint32 *) ( ((char *) src_bitmap->Bits()) +  ((sj * src_bitmap->BytesPerRow()) + (si * sizeof(uint32)))));

      si++;
      di--;          
    }    
    sj++;
    dj--;
  }
}
void SpoolFile::FlipImage180ToBW(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  rgb_color color;
  int32 si,sj,di,dj;
  int32 w = (int32) des_bitmap->Bounds().Width();
  int32 h = (int32) des_bitmap->Bounds().Height();
    
  sj = 0;
  dj = h - 1;
  
  while ((sj < h) && (dj >= 0))
  {
    si = 0;
    di = w - 1;
    
    while ((si < w) && (di >= 0))
    {
      memcpy(&color,
             ((char *) ( ((char *) src_bitmap->Bits()) +  ((sj * src_bitmap->BytesPerRow()) + (si * sizeof(uint32))))),
             sizeof(color));
      
      ToBW(color);
      memcpy(      
             ((char *) ( ((char *) des_bitmap->Bits()) +  ((dj * des_bitmap->BytesPerRow()) + (di * sizeof(uint32))))),
             &color,
             sizeof(color));
      si++;
      di--;          
    }    
    sj++;
    dj--;
  }
}
void SpoolFile::FlipImage90(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  int32 j,i;
  int32 w = (int32) src_bitmap->Bounds().Width();
  int32 h = (int32) src_bitmap->Bounds().Height();
 
  j = 0;
  while (j < h)
  {
    i = 0;
    while (i < w)
    {
      *((uint32 *) ( ((char *) des_bitmap->Bits()) + ((i * des_bitmap->BytesPerRow()) + ((h - j) * sizeof(uint32)) )))
      =
      *((uint32 *) ( ((char *) src_bitmap->Bits()) + ((j * src_bitmap->BytesPerRow()) + (i * sizeof(uint32)) )));

      i++;
    }  
    j++;
  }  
} 
void SpoolFile::FlipImage90ToBW(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  rgb_color color;
  int32 j,i;
  int32 w = (int32) src_bitmap->Bounds().Width();
  int32 h = (int32) src_bitmap->Bounds().Height();
 
  j = 0;
  while (j < h)
  {
    i = 0;
    while (i < w)
    {
      memcpy(
             &color,
             ((char *) ( ((char *) src_bitmap->Bits()) + ((j * src_bitmap->BytesPerRow()) + (i * sizeof(uint32)) ))),
             sizeof(color)
            );
      ToBW(color);
      memcpy(
             ((char *) ( ((char *) des_bitmap->Bits()) + ((i * des_bitmap->BytesPerRow()) + ((h - j) * sizeof(uint32)) ))),
             &color,
             sizeof(color)
            );
      i++;
    }  
    j++;
  }  
}

void SpoolFile::FlipImage270(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  int32 j,i;
  int32 w = (int32) src_bitmap->Bounds().Width();
  int32 h = (int32) src_bitmap->Bounds().Height();
 
  j = 0;
  
  while (j < h)
  {
    i = 0;
    while (i < w)
    {
      *((uint32 *) ( ((char *) des_bitmap->Bits()) + ( ((w - i) * des_bitmap->BytesPerRow()) + (j * sizeof(uint32)) )))
      =
      *((uint32 *) ( ((char *) src_bitmap->Bits()) +  ( (j *  src_bitmap->BytesPerRow()) + (i * sizeof(uint32)) )));
      i++;     
    }
    j++;
  }
}
void SpoolFile::FlipImage270ToBW(BBitmap *des_bitmap,BBitmap *src_bitmap)
{
  rgb_color color;
  int32 j,i;
  int32 w = (int32) src_bitmap->Bounds().Width();
  int32 h = (int32) src_bitmap->Bounds().Height();
 
  j = 0;
  
  while (j < h)
  {
    i = 0;
    while (i < w)
    {

      memcpy(
             &color,
             ((char *) ( ((char *) src_bitmap->Bits()) +  ( (j *  src_bitmap->BytesPerRow()) + (i * sizeof(uint32)) ))),
             sizeof(color)
            );

      ToBW(color);

      memcpy(
             ((char *) ( ((char *) des_bitmap->Bits()) + ( ((w - i) * des_bitmap->BytesPerRow()) + (j * sizeof(uint32)) ))),
             &color,
             sizeof(color)
            );
      i++;     
    }
    j++;
  }
}
PictureSpool *SpoolFile::GetPicture(int pagenum)
{
  return (PictureSpool *) fpiclist->ItemAt(pagenum);
}
int32 SpoolFile::CountPictures()
{
  return fpiclist->CountItems();
}

/* =========================================================================== */

/* =========================================================================== */
PreviewSpoolBitmap::PreviewSpoolBitmap(SpoolFile *spool_file)
:fbitmap(NULL)
{
  BRect paper_rect,bitmap_rect;
  
  paper_rect = spool_file->PaperRect();

  if (spool_file->Portrait())
  {
    // calculate bitmap rect;
    bitmap_rect = BRect(0,0,paper_rect.Width(),paper_rect.Height());
  }
  else
  {
    // calculate bitmap rect;
    bitmap_rect = BRect(0,0,paper_rect.Height(),paper_rect.Width());
  }
  
  // create fbitmap
  fbitmap = new BBitmap(bitmap_rect,B_RGBA32,true,false);
}
PreviewSpoolBitmap::~PreviewSpoolBitmap()
{
  if (fbitmap != NULL)
  delete fbitmap;
}
BBitmap *PreviewSpoolBitmap::Bitmap()const
{
  return fbitmap;
}
/* =========================================================================== */
