#include "PartialPreviewView.h"
PartialPreviewView::PartialPreviewView(BRect frame,BBitmap *bitmap)
:BView(frame,"PartialPreviewView",B_FOLLOW_ALL_SIDES, B_WILL_DRAW|B_FRAME_EVENTS)
,fbitmap(bitmap)
{

}
void PartialPreviewView::Draw(BRect draw_rect)
{
  if (fbitmap != NULL)
  DrawBitmap(fbitmap,BPoint(0,0));
}
void PartialPreviewView::AttachedToWindow(void)
{
  FixupScrollbars();
  Invalidate();
}
void PartialPreviewView::FrameResized(float width,float height)
{
  FixupScrollbars();
  Invalidate();
}
void PartialPreviewView::SetBitmap(BBitmap *bitmap)
{
  fbitmap = bitmap;
}
BBitmap *PartialPreviewView::Bitmap()
{
  return fbitmap;
}
void PartialPreviewView::Update()
{
  FixupScrollbars();
  Invalidate();
}
void PartialPreviewView::GetMaxSize(float *maxx, float *maxy)	
{
  BWindow *mywin = Window();
  float mx, my;
  mx = my = 0.0f;
  if(mywin && mywin->Lock())
  {
    if (fbitmap != NULL)
    {
      BRect imagebounds = fbitmap->Bounds();
      mx = imagebounds.right;
      my = imagebounds.bottom;
    }
    else
    {
      mx = 0;
      my = 0;
    }
    mywin->Unlock();
  }
  if(maxx) 
  *maxx = mx;
  if(maxy) 
  *maxy = my;
}
void PartialPreviewView::FixupScrollbars(void)
{
  BRect bound;
  BScrollBar *bar;
  bound = Bounds();
  float myPixelWidth = bound.Width();
  float myPixelHeight = bound.Height();
  float maxWidth = 1,maxHeight = 1;

  if(fbitmap != NULL)
  GetMaxSize(&maxWidth, &maxHeight);
		
  float propW = myPixelWidth/maxWidth;
  float propH = myPixelHeight/maxHeight;
	
  float rangeW = maxWidth - myPixelWidth;
  float rangeH = maxHeight - myPixelHeight;
  if (rangeW < 0) 
  rangeW = 0;
  if (rangeH < 0)
  rangeH = 0;


  if ((bar=ScrollBar(B_HORIZONTAL))!=NULL) 
  {
	bar->SetProportion(propW);
	bar->SetRange(0,rangeW);
	// Steps are 1/8 visible window for small steps
	//   and 1/2 visible window for large steps
	bar->SetSteps(myPixelWidth / 8.0, myPixelWidth / 2.0);
  } 

  if ((bar=ScrollBar(B_VERTICAL))!=NULL) 
  {
	bar->SetProportion(propH);
	bar->SetRange(0,rangeH);
	// Steps are 1/8 visible window for small steps
	//   and 1/2 visible window for large steps
	bar->SetSteps(myPixelHeight / 8.0, myPixelHeight / 2.0);
  }  
}
